# Copyright (C) 2025, Red Hat Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author: Carlos Garnacho <carlosg@gnome.org>

"""
Test `localsearch inhibit` subcommand
"""

import gi

from gi.repository import Gio

from typing import *
import dataclasses
import pathlib
import os
import subprocess
import time
import sys

import configuration
import fixtures
import shutil

class TestCli(fixtures.TrackerCommandLineTestCase):
    def test_inhibit(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        with subprocess.Popen(
                ["localsearch", "inhibit", "cat"],
                stdin=subprocess.PIPE,
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL) as proc:
            paused = False
            n_tries = 0
            while not paused:
                [apps, reasons] = self.miner_fs.miner_fs.GetPauseDetails()
                if len(apps) > 0:
                    paused = True
                else:
                    n_tries += 1
                    if n_tries > 30:
                        proc.stdin.close()
                        raise RuntimeError("did not pause")
                    time.sleep(1)

            # Check that the command appears in the list
            output = self.run_cli(["localsearch", "inhibit", "--list"])
            self.assertIn("cat", output)

            file = datadir.joinpath("text/mango.txt")
            target = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file)))
            shutil.copy(file, self.indexed_dir)

            # Wait some time and check that the resource is still missing
            time.sleep(3)
            self.assertResourceMissing(self.uri(target))

            # Check that the file gets indexed after the inhibition is gone
            with self.await_document_inserted(target):
                # Close stdin to end the cat process
                proc.stdin.close()

            # List should now be empty
            output = self.run_cli(["localsearch", "inhibit", "--list"])
            self.assertEqual("", output)

    def test_inhibit_terminated(self):
        """Test that terminating `localsearch inhibit` resumes indexing"""
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        with subprocess.Popen(
                ["localsearch", "inhibit", "cat"],
                stdin=subprocess.PIPE,
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL) as proc:
            paused = False
            n_tries = 0
            while not paused:
                [apps, reasons] = self.miner_fs.miner_fs.GetPauseDetails()
                if len(apps) > 0:
                    paused = True
                else:
                    n_tries += 1
                    if n_tries > 30:
                        proc.stdin.close()
                        raise RuntimeError("did not pause")
                    time.sleep(1)

            # Check that the command appears in the list
            output = self.run_cli(["localsearch", "inhibit", "--list"])
            self.assertIn("cat", output)

            # Check that status is paused
            output = self.run_cli(["localsearch", "status"])
            self.assertIn("Indexer is paused", output)

            file = datadir.joinpath("text/mango.txt")
            target = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file)))
            shutil.copy(file, self.indexed_dir)

            # Wait some time and check that the resource is still missing
            time.sleep(3)
            self.assertResourceMissing(self.uri(target))

            # Check that the file gets indexed after the inhibition is gone
            with self.await_document_inserted(target):
                # Terminate the process
                proc.terminate()

            # List should now be empty
            output = self.run_cli(["localsearch", "inhibit", "--list"])
            self.assertEqual("", output)

    def test_inhibit_wrongargs(self):
        err = None
        out = None
        try:
            # Pass unknown arg
            out = self.run_cli(["localsearch", "inhibit", "--asdf"])
        except Exception as e:
            err = str(e)
        finally:
            self.assertIn("--asdf", err);
            self.assertIsNone(out)

    def test_inhibit_noargs(self):
        err = None
        out = None
        try:
            # Pass no command
            out = self.run_cli(["localsearch", "inhibit"])
        except Exception as e:
            err = str(e)
        finally:
            self.assertIn("Usage", err);
            self.assertIsNone(out)

if __name__ == "__main__":
    fixtures.tracker_test_main()



#
# Copyright (C) 2010, Nokia <ivan.frade@nokia.com>
# Copyright (C) 2018-2019, Sam Thursfield <sam@afuera.me.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
"""
For a collection of files, call the extractor and check that the expected
metadata is extracted. Load dynamically the test information from a data
directory (containing xxx.expected files)
"""

import gi

gi.require_version("Tsparql", "3.0")
from gi.repository import Gio
from gi.repository import Tsparql

import json
import logging
import os
import shutil
import sys
import tempfile
import unittest as ut

import configuration as cfg
import fixtures


class GenericExtractionTestCase(fixtures.TrackerExtractTestCase):
    """
    Test checks if the tracker extractor is able to retrieve metadata
    """

    def __init__(self, methodName="runTest", descfile=None):
        """
        Descfile is the description file in a relative path
        """
        self.descfile = descfile
        try:
            with open(descfile) as f:
                self.spec = json.load(f)
        except ValueError as e:
            self.fail("Error loading %s: %s" % (descfile, e))

        # Add a method to the class called after the description file
        methodName = descfile.lower()[: -len(".expected")].replace(" ", "_")[-60:]

        if self.spec["test"].get("ExpectedFailure", False):
            setattr(self, methodName, self.expected_failure_test_extraction)
        else:
            setattr(self, methodName, self.generic_test_extraction)

        super(GenericExtractionTestCase, self).__init__(methodName)

    def __get_bugnumber(self):
        return self.spec["test"].get("Bugzilla")

    def validate_trig(self, trig):
        """Create a temporary database and run the given RDF data.

        This gives us a smoke test to detect any situation where the
        extractor generates invalid RDF.
        """
        cancellable = None
        ontology = Tsparql.sparql_get_ontology_nepomuk()
        db = Tsparql.SparqlConnection.new(
            Tsparql.SparqlConnectionFlags.NONE,
            None,  # create in-memory database,
            ontology,
            cancellable,
        )

        batch = db.create_batch()
        istream = Gio.MemoryInputStream.new_from_data(bytearray(trig, 'utf-8'))
        batch.add_rdf(Tsparql.DeserializeFlags.NONE, Tsparql.RdfFormat.TRIG, 'tracker:FileSystem', istream)
        self.assertTrue(batch.execute(cancellable))

    def generic_test_extraction(self):
        abs_description = os.path.abspath(self.descfile)

        # Filename contains the file to extract, in a relative path to the description file
        desc_root, desc_file = os.path.split(abs_description)

        filename_to_extract = self.spec["test"]["Filename"]
        self.file_to_extract = os.path.join(desc_root, filename_to_extract)

        tmpdir = tempfile.mkdtemp(prefix="tracker-extract-test-")
        try:
            extra_env = cfg.test_environment(tmpdir)
            if "metadata" in self.spec:
                jsonld = fixtures.get_tracker_extract_output(
                    extra_env, self.file_to_extract, output_format="json-ld"
                )
                self.__assert_extraction_ok(jsonld)

                trig = fixtures.get_tracker_extract_output(
                    extra_env, self.file_to_extract, output_format="trig"
                )
                self.validate_trig(trig)
            else:
                # No output data is expected for this file
                jsonld = None
                try:
                    jsonld = fixtures.get_tracker_extract_output(
                        extra_env, self.file_to_extract, output_format="json-ld"
                    )
                    self.assertEqual(jsonld["@graph"], [])
                except RuntimeError as re:
                    print ('# Got expected error message: %s' % str(re).replace('\n', '\\n'))
        finally:
            shutil.rmtree(tmpdir, ignore_errors=True)

    @ut.expectedFailure
    def expected_failure_test_extraction(self):
        self.generic_test_extraction()

        if self.__get_bugnumber():
            raise Exception(
                "Unexpected success. Maybe bug: "
                + self.__get_bugnumber()
                + " has been fixed?"
            )
        else:
            raise Exception("Unexpected success. Check " + self.rel_description)

    def __assert_extraction_ok(self, result):
        try:
            self.assert_extract_result_matches_spec(
                self.spec["metadata"], result, self.file_to_extract, self.descfile
            )
        except AssertionError:
            print("\ntracker-extract returned: %s" % json.dumps(result, indent=4))
            raise


def run_suite(suite):
    if cfg.tests_verbose():
        # Output all logs to stderr
        logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)

    if cfg.tap_protocol_enabled():
        try:
            from tap import TAPTestRunner

            runner = TAPTestRunner()
            runner.set_stream(True)
        except ImportError as e:
            log.error("No TAP test runner found: %s", e)
            raise
    else:
        runner = ut.TextTestRunner(verbosity=1)

    result = runner.run(suite)
    sys.exit(not result.wasSuccessful())


def run_all():
    ##
    # Traverse the TEST_DATA_PATH directory looking for .description files
    # Add a new TestCase to the suite per .description file and run the suite.
    #
    # Is we do this inside a single TestCase an error in one test would stop the whole
    # testing.
    ##
    if os.path.exists(os.getcwd() + "/test-extraction-data"):
        # Use local directory if available
        TEST_DATA_PATH = os.getcwd() + "/test-extraction-data"
    else:
        TEST_DATA_PATH = os.path.join(
            cfg.DATADIR, "tracker-tests", "test-extraction-data"
        )
    print("Loading test descriptions from", TEST_DATA_PATH)
    extractionTestSuite = ut.TestSuite()
    for root, dirs, files in os.walk(TEST_DATA_PATH):
        descriptions = [os.path.join(root, f) for f in files if f.endswith("expected")]
        for descfile in descriptions:
            tc = GenericExtractionTestCase(descfile=descfile)
            extractionTestSuite.addTest(tc)

    run_suite(extractionTestSuite)


def run_one(filename):
    ##
    # Run just one .description file
    ##
    description = os.path.join(os.getcwd(), filename)

    extractionTestSuite = ut.TestSuite()
    tc = GenericExtractionTestCase(descfile=description)
    extractionTestSuite.addTest(tc)

    run_suite(extractionTestSuite)


try:
    if len(sys.argv) == 2:
        run_one(sys.argv[1])
    elif len(sys.argv) == 1:
        run_all()
    else:
        raise RuntimeError("Too many arguments.")
except RuntimeError as e:
    sys.stderr.write("ERROR: %s\n" % e)
    sys.exit(1)



# Copyright (C) 2025, Red Hat Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author: Carlos Garnacho <carlosg@gnome.org>

"""
Test `localsearch info` subcommand
"""

import gi
gi.require_version("Tsparql", "3.0")
from gi.repository import Gio, GLib
from gi.repository import Tsparql

from typing import *
import dataclasses
import pathlib
import os
import re
import time

import configuration
import fixtures
import shutil

import configuration as cfg

class TestCli(fixtures.TrackerCommandLineTestCase):
    def validate_trig(self, trig):
        """Create a temporary database and run the given RDF data.

        This gives us a smoke test to detect any situation where the
        extractor generates invalid RDF.
        """
        cancellable = None
        ontology = Tsparql.sparql_get_ontology_nepomuk()
        db = Tsparql.SparqlConnection.new(
            Tsparql.SparqlConnectionFlags.NONE,
            None,  # create in-memory database,
            ontology,
            cancellable,
        )

        batch = db.create_batch()
        istream = Gio.MemoryInputStream.new_from_data(bytearray(trig, 'utf-8'))
        batch.add_rdf(Tsparql.DeserializeFlags.NONE, Tsparql.RdfFormat.TRIG, 'tracker:FileSystem', istream)
        self.assertTrue(batch.execute(cancellable))

    def test_info(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        # Copy a file and wait for it to be indexed, in order to ensure idle state
        file = datadir.joinpath("text/mango.txt")
        target = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file)))
        with self.await_document_inserted(target):
            shutil.copy(file, self.indexed_dir)

        output = self.run_cli(["localsearch", "info", self.uri(target)])
        self.assertIn("nfo:PlainTextDocument", output)
        self.assertIn("mango.txt", output)

    def test_info_plain_text(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        # Copy a file and wait for it to be indexed, in order to ensure idle state
        file = datadir.joinpath("text/mango.txt")
        target = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file)))
        with self.await_document_inserted(target):
            shutil.copy(file, self.indexed_dir)

        output = self.run_cli(["localsearch", "info", target, "--plain-text-content"])
        self.assertIn("nfo:PlainTextDocument", output)
        self.assertIn("mango.txt", output)
        self.assertIn("monkey", output)

    def test_info_full_namespaces(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        # Copy a file and wait for it to be indexed, in order to ensure idle state
        file = datadir.joinpath("text/mango.txt")
        target = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file)))
        with self.await_document_inserted(target):
            shutil.copy(file, self.indexed_dir)

        output = self.run_cli(["localsearch", "info", target, "--full-namespaces"])
        self.assertIn("http://tracker.api.gnome.org/ontology/v3/nfo#PlainTextDocument", output)
        self.assertNotIn("nfo:PlainTextDocument", output)

    def test_info_eligible(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        # Copy a file and wait for it to be indexed, in order to ensure idle state
        file = datadir.joinpath("text/mango.txt")
        target = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file)))
        with self.await_document_inserted(target):
            shutil.copy(file, self.indexed_dir)

        output = self.run_cli(["localsearch", "info", "--eligible", target])
        self.assertIn("currently exists", output)
        self.assertIn("File is eligible", output)

    def test_info_eligible_nonexistent(self):
        target = self.path("test-monitored/non-existent.txt")
        output = self.run_cli(["localsearch", "info", "--eligible", target])
        self.assertIn("does not exist", output)
        self.assertIn("File is eligible", output)

    def test_info_non_eligible(self):
        # Non-existent file
        target = "/non-existent.txt"
        output = self.run_cli(["localsearch", "info", "--eligible", target])
        self.assertIn("does not exist", output)
        self.assertIn("not an indexed folder", output)

        # Hidden file
        target = self.path("test-monitored/.hidden.txt")
        with open(target, "w") as f:
            f.write("Foo bar baz")
        output = self.run_cli(["localsearch", "info", "--eligible", target])
        self.assertIn("currently exist", output)
        self.assertIn("hidden file", output)

        # File ineligible by glob filter
        target = self.path("test-monitored/backup.txt~")
        with open(target, "w") as f:
            f.write("Foo bar baz")
        output = self.run_cli(["localsearch", "info", "--eligible", target])
        self.assertIn("currently exist", output)
        self.assertIn("based on filters", output)

        # Directory ineligible by glob filter
        target = pathlib.Path(self.path("test-monitored/lost+found"))
        target.mkdir(parents=True, exist_ok=True)
        output = self.run_cli(["localsearch", "info", "--eligible", target])
        self.assertIn("currently exist", output)
        self.assertIn("based on filters", output)

        # File filtered by parent-dir filter
        nomedia = self.path("test-monitored/.nomedia")
        with open(nomedia, "w") as f:
            f.write("")
        target = self.path("test-monitored/file.txt")
        with open(target, "w") as f:
            f.write("Foo bar baz")
        output = self.run_cli(["localsearch", "info", "--eligible", target])
        self.assertIn("currently exist", output)
        self.assertIn("based on content filters", output)


    def test_info_non_eligible_parent(self):
        # Hidden parent folder
        target = pathlib.Path(self.path("test-monitored/.folder/file.txt"))
        target.parent.mkdir(parents=True, exist_ok=True)
        target.write_text('foo bar baz')
        output = self.run_cli(["localsearch", "info", "--eligible", target])
        self.assertIn("Parent directory", output)
        self.assertIn("hidden file", output)

        # Parent folder ineligible by filter
        target = pathlib.Path(self.path("test-monitored/lost+found/file.txt"))
        target.parent.mkdir(parents=True, exist_ok=True)
        target.write_text('foo bar baz')
        output = self.run_cli(["localsearch", "info", "--eligible", target])
        self.assertIn("Parent directory", output)
        self.assertIn("based on filters", output)

        # Parent folder ineligible by content filter
        target = pathlib.Path(self.path("test-monitored/a/b/file.txt"))
        target.parent.mkdir(parents=True, exist_ok=True)
        target.write_text('foo bar baz')
        nomedia = pathlib.Path(self.path("test-monitored/a/.nomedia"))
        nomedia.write_text('')
        output = self.run_cli(["localsearch", "info", "--eligible", target])
        self.assertIn("Parent directory", output)
        self.assertIn("based on content filters", output)

    def test_info_nonexistent(self):
        err = None
        out = None
        try:
            target = pathlib.Path(os.path.join(self.indexed_dir, 'nonexistent.txt'))
            out = self.run_cli(["localsearch", "info", target])
        except Exception as e:
            err = str(e)
        finally:
            self.assertIn("No metadata available for that URI", err)
            self.assertIsNone(out)

    def test_info_noargs(self):
        err = None
        out = None
        try:
            out = self.run_cli(["localsearch", "info"])
        except Exception as e:
            err = str(e)
        finally:
            self.assertIn("Usage", err)
            self.assertIsNone(out)

    def test_info_wrongargs(self):
        err = None
        out = None
        try:
            # Pass unknown arg
            out = self.run_cli(["localsearch", "info", "--asdf"])
        except Exception as e:
            err = str(e)
        finally:
            self.assertIn("--asdf", err);
            self.assertIsNone(out)

    def test_info_rdf(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        # Move a file and wait for it to be indexed, in order to ensure idle state
        target = pathlib.Path(os.path.join(self.indexed_dir, "mango.txt"))
        with self.await_document_inserted(target):
            GLib.file_set_contents(target.as_posix(), b"monkey")

        output = self.run_cli(["localsearch", "info", target, "--output-format", "trig"])
        self.assertIn("@prefix", output)
        self.assertIn("mango.txt", output)
        self.assertIn("monkey", output)
        self.validate_trig(output)

    def test_info_rdf_wrongarg(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        # Copy a file and wait for it to be indexed, in order to ensure idle state
        file = datadir.joinpath("text/mango.txt")
        target = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file)))
        with self.await_document_inserted(target):
            shutil.copy(file, self.indexed_dir)

        err = None
        out = None
        try:
            output = self.run_cli(["localsearch", "info", target, "--output-format", "asdf"])
        except Exception as e:
            err = str(e)
        finally:
            self.assertIn("Unsupported serialization format", err);
            self.assertIsNone(out)

if __name__ == "__main__":
    fixtures.tracker_test_main()



# Copyright (C) 2010, Nokia (ivan.frade@nokia.com)
# Copyright (C) 2019-2020, Sam Thursfield (sam@afuera.me.uk)
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA  02110-1301, USA.

#
# TODO:
#     These tests are for files... we need to write them for folders!
#

"""
Monitor a test directory and copy/move/remove/update files and folders there.
Check the basic data of the files is updated accordingly in tracker.
"""

import itertools
import logging
import os
import shutil
import time
import unittest as ut

import configuration as cfg
import fixtures


log = logging.getLogger(__name__)

DEFAULT_TEXT = "Some stupid content, to have a test file"


class MinerCrawlTest(fixtures.TrackerMinerTest):
    """
    Tests crawling and monitoring of configured content locations.
    """

    def setUp(self):
        # We must create the test data before the miner does its
        # initial crawl, or it may miss some files due
        # https://gitlab.gnome.org/GNOME/tracker-miners/issues/79.
        monitored_files = self.create_test_data()

        try:
            # Start the miner.
            fixtures.TrackerMinerTest.setUp(self)

            for tf in monitored_files:
                url = self.uri(tf)
                self.tracker.ensure_resource(
                    fixtures.DOCUMENTS_GRAPH,
                    f"a nfo:Document ; nie:isStoredAs <{url}>",
                    timeout=cfg.AWAIT_TIMEOUT,
                )
        except Exception:
            cfg.remove_monitored_test_dir(self.workdir)
            raise

        logging.info("%s.setUp(): complete", self)

    def create_test_data(self):
        monitored_files = [
            "test-monitored/file1.txt",
            "test-monitored/dir1/file2.txt",
            "test-monitored/dir1/dir2/file3.txt",
        ]

        unmonitored_files = ["test-no-monitored/file0.txt"]

        for tf in itertools.chain(monitored_files, unmonitored_files):
            testfile = self.path(tf)
            os.makedirs(os.path.dirname(testfile), exist_ok=True)
            with open(testfile, "w") as f:
                f.write(DEFAULT_TEXT)

        return monitored_files

    def __get_text_documents(self):
        return self.tracker.query(
            """
          SELECT DISTINCT ?url WHERE {
              ?u a nfo:TextDocument ;
                 nie:isStoredAs/nie:url ?url.
          }
          """
        )

    def __get_parent_urn(self, filepath):
        result = self.tracker.query(
            """
          SELECT DISTINCT ?p WHERE {
              ?u a nfo:FileDataObject ;
                 nie:url \"%s\" ;
                 nfo:belongsToContainer ?p
          }
          """
            % (self.uri(filepath))
        )
        self.assertEqual(len(result), 1)
        return result[0][0]

    def __get_file_urn(self, filepath):
        result = self.tracker.query(
            """
          SELECT DISTINCT ?ia WHERE {
              ?u a nfo:FileDataObject ;
                 nie:interpretedAs ?ia ;
                 nie:url \"%s\" .
          }
          """
            % (self.uri(filepath))
        )
        self.assertEqual(len(result), 1)
        return result[0][0]

    """
    Boot the miner with the correct configuration and check everything is fine
    """

    def test_01_initial_crawling(self):
        """
        The precreated files and folders should be there
        """
        result = self.__get_text_documents()
        self.assertEqual(len(result), 3)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/file2.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/dir2/file3.txt"), unpacked_result)

        # We don't check (yet) folders, because Applications module is injecting results

    # class copy(TestUpdate):
    # FIXME all tests in one class because the miner-fs restarting takes some time (~5 sec)
    # Maybe we can move the miner-fs initialization to setUpModule and then move these
    # tests to different classes

    def test_02_copy_from_unmonitored_to_monitored(self):
        """
        Copy an file from unmonitored directory to monitored directory
        and verify if data base is updated accordingly
        """
        source = os.path.join(self.workdir, "test-no-monitored", "file0.txt")
        dest = os.path.join(self.workdir, "test-monitored", "file0.txt")

        with self.await_document_inserted(dest) as resource:
            shutil.copyfile(source, dest)
        dest_id = resource.id

        # Verify if miner indexed this file.
        result = self.__get_text_documents()
        self.assertEqual(len(result), 4)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/file2.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/dir2/file3.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/file0.txt"), unpacked_result)

        # Clean the new file so the test directory is as before
        with self.tracker.await_delete(
            fixtures.DOCUMENTS_GRAPH, dest_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            os.remove(dest)

    def test_03_copy_from_monitored_to_unmonitored(self):
        """
        Copy an file from a monitored location to an unmonitored location
        Nothing should change
        """

        # Copy from monitored to unmonitored
        source = os.path.join(self.workdir, "test-monitored", "file1.txt")
        dest = os.path.join(self.workdir, "test-no-monitored", "file1.txt")
        shutil.copyfile(source, dest)

        time.sleep(1)
        # Nothing changed
        result = self.__get_text_documents()
        self.assertEqual(len(result), 3, "Results:" + str(result))
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/file2.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/dir2/file3.txt"), unpacked_result)

        # Clean the file
        os.remove(dest)

    def test_04_copy_from_monitored_to_monitored(self):
        """
        Copy a file between monitored directories
        """
        source = os.path.join(self.workdir, "test-monitored", "file1.txt")
        dest = os.path.join(
            self.workdir, "test-monitored", "dir1", "dir2", "file-test04.txt"
        )

        with self.await_document_inserted(dest) as resource:
            shutil.copyfile(source, dest)
        dest_id = resource.id

        result = self.__get_text_documents()
        self.assertEqual(len(result), 4)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/file2.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/dir2/file3.txt"), unpacked_result)
        self.assertIn(
            self.uri("test-monitored/dir1/dir2/file-test04.txt"), unpacked_result
        )

        with self.tracker.await_delete(
            fixtures.DOCUMENTS_GRAPH, dest_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            os.remove(dest)

        self.assertEqual(3, self.tracker.count_instances("nfo:TextDocument"))

    @ut.skip("https://gitlab.gnome.org/GNOME/tracker-miners/issues/56")
    def test_05_move_from_unmonitored_to_monitored(self):
        """
        Move a file from unmonitored to monitored directory
        """
        source = os.path.join(self.workdir, "test-no-monitored", "file0.txt")
        dest = os.path.join(self.workdir, "test-monitored", "dir1", "file-test05.txt")

        with self.await_document_inserted(dest) as resource:
            shutil.move(source, dest)
        dest_id = resource.id

        result = self.__get_text_documents()
        self.assertEqual(len(result), 4)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/file2.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/dir2/file3.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/file-test05.txt"), unpacked_result)

        with self.tracker.await_delete(
            fixtures.DOCUMENTS_GRAPH, dest_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            os.remove(dest)

        self.assertEqual(3, self.tracker.count_instances("nfo:TextDocument"))

    ## """ move operation and tracker-miner response test cases """
    # class move(TestUpdate):

    def test_06_move_from_monitored_to_unmonitored(self):
        """
        Move a file from monitored to unmonitored directory
        """
        source = self.path("test-monitored/dir1/file2.txt")
        dest = self.path("test-no-monitored/file2.txt")
        source_id = self.tracker.get_content_resource_id(self.uri(source))
        with self.tracker.await_delete(
            fixtures.DOCUMENTS_GRAPH, source_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            shutil.move(source, dest)

        result = self.__get_text_documents()
        self.assertEqual(len(result), 2)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/dir2/file3.txt"), unpacked_result)

        with self.await_document_inserted(source):
            shutil.move(dest, source)
        self.assertEqual(3, self.tracker.count_instances("nfo:TextDocument"))

    def test_07_move_from_monitored_to_monitored(self):
        """
        Move a file between monitored directories
        """

        source = self.path("test-monitored/dir1/file2.txt")
        dest = self.path("test-monitored/file2.txt")

        source_dir_urn = self.__get_file_urn(os.path.dirname(source))
        parent_before = self.__get_parent_urn(source)
        self.assertEqual(source_dir_urn, parent_before)

        resource_id = self.tracker.get_content_resource_id(url=self.uri(source))
        with self.await_document_uri_change(resource_id, source, dest):
            shutil.move(source, dest)

        # Checking fix for NB#214413: After a move operation, nfo:belongsToContainer
        # should be changed to the new one
        dest_dir_urn = self.__get_file_urn(os.path.dirname(dest))
        parent_after = self.__get_parent_urn(dest)
        self.assertNotEqual(parent_before, parent_after)
        self.assertEqual(dest_dir_urn, parent_after)

        result = self.__get_text_documents()
        self.assertEqual(len(result), 3)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/file2.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/dir2/file3.txt"), unpacked_result)

        # Restore the file
        with self.await_document_uri_change(resource_id, dest, source):
            shutil.move(dest, source)

        result = self.__get_text_documents()
        self.assertEqual(len(result), 3)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/dir1/file2.txt"), unpacked_result)

    def test_08_deletion_single_file(self):
        """
        Delete one of the files
        """
        victim = self.path("test-monitored/dir1/file2.txt")
        victim_id = self.tracker.get_content_resource_id(self.uri(victim))
        with self.tracker.await_delete(
            fixtures.DOCUMENTS_GRAPH, victim_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            os.remove(victim)

        result = self.__get_text_documents()
        self.assertEqual(len(result), 2)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/dir2/file3.txt"), unpacked_result)

        # Restore the file
        with self.await_document_inserted(victim):
            with open(victim, "w") as f:
                f.write("Don't panic, everything is fine")

    def test_09_deletion_directory(self):
        """
        Delete a directory
        """
        victim = self.path("test-monitored/dir1")

        file_inside_victim_url = self.uri(os.path.join(victim, "file2.txt"))
        file_inside_victim_id = self.tracker.get_content_resource_id(
            file_inside_victim_url
        )

        with self.tracker.await_delete(
            fixtures.DOCUMENTS_GRAPH, file_inside_victim_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            shutil.rmtree(victim)

        # We have 2 text files inside the directory, but only look for one, assert that
        # the other file is gone as well.
        file_inside_victim_url2 = self.uri(os.path.join(victim, "dir2/file3.txt"))
        counter = 0
        while counter < 10 and self.tracker.ask(
            "ASK { <%s> a nfo:FileDataObject }" % (file_inside_victim_url2)
        ):
            counter += 1
            time.sleep(1)

        result = self.__get_text_documents()
        self.assertEqual(result, [[self.uri("test-monitored/file1.txt")]])

        # Restore the dirs
        os.makedirs(self.path("test-monitored/dir1"))
        os.makedirs(self.path("test-monitored/dir1/dir2"))
        for f in [
            "test-monitored/dir1/file2.txt",
            "test-monitored/dir1/dir2/file3.txt",
        ]:
            filename = self.path(f)
            with self.await_document_inserted(filename):
                with open(filename, "w") as f:
                    f.write("Don't panic, everything is fine")

        # Check everything is fine
        result = self.__get_text_documents()
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/file2.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/dir2/file3.txt"), unpacked_result)
        self.assertEqual(len(result), 3)

    def test_10_folder_update(self):
        """
        Check that updating a folder updates nfo:belongsToContainer on its children
        """

        directory_uri = self.uri("test-monitored")
        directory = self.path("test-monitored")
        document = self.path("test-monitored/unrelated.txt")
        urn = self.__get_file_urn(directory)

        with self.await_document_inserted(document) as resource:
            # Force an update on the monitored folder, it needs both
            # a explicit request, and an attribute change (obtained
            # indirectly by the new file)
            with open(document, "w") as f:
                f.write(DEFAULT_TEXT)
            self.miner_fs.index_location(directory_uri, [], [])

        new_urn = self.__get_file_urn(directory)
        # Ensure that children remain consistent, old and new ones
        self.assertEqual(
            new_urn, self.__get_parent_urn(self.path("test-monitored/file1.txt"))
        )
        self.assertEqual(
            self.__get_parent_urn(document),
            self.__get_parent_urn(self.path("test-monitored/file1.txt")),
        )

    def test_11_move_from_visible_to_hidden(self):
        """
        Move a file from monitored to unmonitored directory
        """
        source = self.path("test-monitored/dir1/file2.txt")
        dest = self.path("test-monitored/dir1/.file2.txt")
        source_id = self.tracker.get_content_resource_id(self.uri(source))
        with self.tracker.await_delete(
            fixtures.DOCUMENTS_GRAPH, source_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            shutil.move(source, dest)

        result = self.__get_text_documents()
        self.assertEqual(len(result), 2)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/dir2/file3.txt"), unpacked_result)

    def test_12_move_from_hidden_to_visible(self):
        """
        Move a file from monitored to unmonitored directory
        """
        source = self.path("test-monitored/dir1/.hidden.txt")
        dest = self.path("test-monitored/dir1/visible.txt")

        with open(source, "w") as f:
            f.write(DEFAULT_TEXT)

        with self.await_document_inserted(dest) as resource:
            shutil.move(source, dest)

        result = self.__get_text_documents()
        self.assertEqual(len(result), 4)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/dir2/file3.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/file2.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/visible.txt"), unpacked_result)

    def test_13_move_directory_from_visible_to_hidden_and_back(self):
        """
        Move a directory from monitored to unmonitored
        """
        file = self.path("test-monitored/dir1/file2.txt")
        source = self.path("test-monitored/dir1")
        dest = self.path("test-monitored/.dir1")

        file_id = self.tracker.get_content_resource_id(self.uri(file))
        with self.tracker.await_delete(
            fixtures.DOCUMENTS_GRAPH, file_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            os.rename(source, dest)

        result = self.__get_text_documents()
        self.assertEqual(len(result), 1)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)

        """
        And move it back to the same place
        """
        with self.await_document_inserted(file) as resource:
            shutil.move(dest, source)

        result = self.__get_text_documents()
        self.assertEqual(len(result), 3)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/dir2/file3.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/file2.txt"), unpacked_result)

    def test_14_trigger_parent_hidden(self):
        """
        Insert a file triggering parent to be hidden
        """
        file = self.path("test-monitored/dir1/.nomedia")
        doc = self.path("test-monitored/dir1/file2.txt")

        doc_id = self.tracker.get_content_resource_id(self.uri(doc))
        with self.tracker.await_delete(
            fixtures.DOCUMENTS_GRAPH, doc_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            with open(file, "w") as f:
                f.write(DEFAULT_TEXT)

        result = self.__get_text_documents()
        self.assertEqual(len(result), 1)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)

    def test_15_monitor_nofollow_symlinks(self):
        """
        Insert a file triggering parent to be hidden
        """
        source = self.path("test-no-monitored")
        dest = self.path("test-monitored/folder")
        deleted = self.path("test-monitored/file1.txt")
        deleted_id = self.tracker.get_content_resource_id(self.uri(deleted))

        with self.tracker.await_delete(
            fixtures.DOCUMENTS_GRAPH, deleted_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            os.symlink(source, dest)
            os.remove(deleted)

        result = self.__get_text_documents()
        self.assertEqual(len(result), 2)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/dir1/file2.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/dir2/file3.txt"), unpacked_result)

        query = "ASK { <%s> a nfo:FileDataObject }" % self.uri(dest)
        result = self.tracker.query(query)
        self.assertEqual(result[0][0], "true")

        query = "ASK { ?u nie:isStoredAs <%s> }" % self.uri(dest)
        result = self.tracker.query(query)
        self.assertEqual(result[0][0], "false")

    def test_16_folder_update_offline(self):
        """
        Check that updating a folder deletes the old nfo:Folder
        """

        directory_uri = self.uri("test-monitored/dir1")
        document_uri = self.uri("test-monitored/dir1/unrelated.txt")
        dest_uri = self.uri("test-monitored/dir1/unrelated2.txt")
        directory = self.path("test-monitored/dir1")
        document = self.path("test-monitored/dir1/unrelated.txt")
        dest = self.path("test-monitored/dir1/unrelated2.txt")
        ie_urn = self.__get_file_urn(directory)

        self.assertResourceExists(ie_urn)

        # Create a dir to overwrite the original first, to guarantee
        # a different inode.
        os.makedirs(self.path("zzz"))

        self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')

        # Replace the dir
        shutil.rmtree(directory)
        shutil.move(self.path("zzz"), directory)

        conn = self.miner_fs.get_sparql_connection()

        # Write document
        with self.await_document_inserted(document) as resource:
            with open(document, "w") as f:
                f.write(DEFAULT_TEXT)

        new_ie_urn = self.__get_file_urn(directory)
        self.assertNotEqual(new_ie_urn, ie_urn)

        self.assertResourceExists(new_ie_urn)
        self.assertResourceExists(document_uri)

        # Check the old URN is not a folder anymore
        query = "ASK { <%s> a nfo:Folder }"
        result = self.tracker.query(query)
        self.assertEqual(result, [['false']])

        query = "SELECT ?u { ?u nie:isStoredAs <%s> }" % self.uri(directory)
        result = self.tracker.query(query)
        self.assertEqual(len(result), 1)
        self.assertEqual(result[0][0], new_ie_urn)

        query = "SELECT ?u { <%s> nie:interpretedAs ?u }" % self.uri(directory)
        result = self.tracker.query(query)
        self.assertEqual(len(result), 1)
        self.assertEqual(result[0][0], new_ie_urn)

        # Move document
        resource_id = self.tracker.get_content_resource_id(document_uri)
        dest = self.path("test-monitored/dir1/unrelated2.txt")
        with self.await_document_uri_change(resource_id, document, dest):
            shutil.move(document, dest)

        self.assertResourceMissing(document_uri)
        self.assertResourceExists(dest_uri)

    def test_17_move_and_replace(self):
        """
        Move a file on top of another, and have another file replace the first
        """
        source = self.path("test-monitored/file1.txt");
        source_urn = self.__get_file_urn(source)
        dest = self.path("test-monitored/dir1/file2.txt");
        dest_urn = self.__get_file_urn(dest)

        source_id = self.tracker.get_content_resource_id(self.uri(source))

        # Move file overwriting another
        with self.await_document_uri_change(source_id, source, dest):
            shutil.move(source, dest)

        result = self.__get_text_documents()
        self.assertEqual(len(result), 2)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/dir1/file2.txt"), unpacked_result)

        # Ensure the file replaced by source changed and matches
        new_dest_urn = self.__get_file_urn(dest)
        self.assertNotEqual(dest_urn, new_dest_urn)
        self.assertEqual(source_urn, new_dest_urn)

        # Write a new document in the former position
        with self.await_document_inserted(source) as resource:
            with open(source, "w") as f:
                f.write("Replacement")

        # Ensure the file created at source location is different
        new_source_urn = self.__get_file_urn(source)
        self.assertNotEqual(source_urn, new_source_urn)

    def test_18_move_and_replace_dir(self):
        """
        Move a directory, and create another directory replacing the first
        """
        source = self.path("test-monitored/dir1");
        source_urn = self.__get_file_urn(source)
        dest = self.path("test-monitored/moved");

        with self.await_insert_dir(dest):
            shutil.move(source, dest)

        with self.await_insert_dir(source):
            os.mkdir(source)

        new_source_urn = self.__get_file_urn(source)
        dest_urn = self.__get_file_urn(dest)

        self.assertEqual(dest_urn, source_urn)
        self.assertNotEqual(new_source_urn, source_urn)

    def test_19_move_and_replace_from_hidden(self):
        """
        Create a hidden file, and make it visible. Then repeat.
        """

        old_text = "I am old!"
        new_text = "I am new!"

        source = self.path("test-monitored/.hidden.txt");
        dest = self.path("test-monitored/nothidden.txt");

        with open(source, "w") as f:
            f.write(old_text)
        with self.await_document_inserted(dest):
            shutil.move(source, dest)

        urn = self.__get_file_urn(dest)

        with open(source, "w") as f:
            f.write(new_text)

        resource_id = self.tracker.get_content_resource_id(self.uri(dest))

        with self.tracker.await_content_update(
            fixtures.DOCUMENTS_GRAPH,
            resource_id,
            f'nie:plainTextContent "{old_text}"',
            f'nie:plainTextContent "{new_text}"',
            timeout=cfg.AWAIT_TIMEOUT,
        ):
            shutil.move(source, dest)

        new_urn = self.__get_file_urn(dest)
        self.assertNotEqual(urn, new_urn)

    def test_20_recover_corruption(self):
        """
        Trick a constraint corruption. See how it fares
        """

        # Insert a blank node with a nie:url
        path = self.path("test-monitored/test.txt")
        uri = self.uri("test-monitored/test.txt")
        self.tracker.update(
            "INSERT DATA { GRAPH tracker:FileSystem { _:u a nfo:FileDataObject ; nie:url '%s' } }" % uri)

        pid = self.sandbox.session_bus.get_connection_unix_process_id_sync(
            'org.freedesktop.LocalSearch3')

        # Create a file on that url, see it fail
        with open(path, "w") as f:
            f.write(DEFAULT_TEXT)

        # Ensure the process is gone
        pid_exists = True
        attempts = 0
        while pid_exists:
            if attempts == 10:
                raise RuntimeError("Took too long for indexer to die")

            try:
                time.sleep(1)
                program = os.readlink("/proc/" + str(pid) + "/exe")
                pid_exists = os.path.basename(program) == "localsearch-3"
                if pid_exists:
                    attempts += 1
            except:
                pid_exists = False

        # Restart the indexer
        with self.await_document_inserted(path) as resource:
            conn = self.miner_fs.get_sparql_connection()

        result = self.__get_text_documents()
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/test.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/file2.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/dir2/file3.txt"), unpacked_result)
        self.assertEqual(len(result), 4)

    def test_21_move_directory_from_hidden_to_visible_and_back(self):
        """
        Move a directory from unmonitored to monitored, and back
        """
        file = self.path("test-monitored/.dir/file.txt")
        dest_file = self.path("test-monitored/dir/file2.txt")
        source = self.path("test-monitored/.dir")
        dest = self.path("test-monitored/dir")

        os.mkdir(source)
        with open(file, "w") as f:
            f.write(DEFAULT_TEXT)

        with self.await_document_inserted(self.path("test-monitored/dir/file.txt")):
            os.rename(source, dest)

        result = self.__get_text_documents()
        # Default test data, plus the new file
        self.assertEqual(len(result), 4)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/dir/file.txt"), unpacked_result)

        file_id = self.tracker.get_content_resource_id(self.uri("test-monitored/dir/file.txt"))
        with self.tracker.await_delete(
            fixtures.DOCUMENTS_GRAPH, file_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            os.rename(dest, source)

        result = self.__get_text_documents()
        # Default test data only
        self.assertEqual(len(result), 3)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)

    def test_22_move_directory_from_hidden_to_visible_and_back_interrupted(self):
        """
        Move a directory from unmonitored to monitored, and back, but
        ensure the crawling operation is interrupted
        """
        file = self.path("test-monitored/.dir/file.txt")
        dest_file = self.path("test-monitored/dir/file2.txt")
        source = self.path("test-monitored/.dir")
        subdir = self.path("test-monitored/.dir/subdir")
        dest = self.path("test-monitored/dir")

        # Queue a substantial amount of changes, so there's larger guarantees
        N_FILES = 5000

        # Create a file to detect that the directory was moved
        os.mkdir(source)
        with open(file, "w") as f:
            f.write(DEFAULT_TEXT)

        # Add a decent amount of files in a subdir
        os.mkdir(subdir)
        for i in range(N_FILES):
            text_file = os.path.join(subdir, "file-%d" % i)
            with open(text_file, "w") as f:
                f.write(DEFAULT_TEXT)

        # Rename the dir and expect our canary file to be indexed
        # This should still be in the middle of operations, since
        # a subdir with enough amount of data has to be crawled
        check_file = self.uri("test-monitored/dir/file.txt")
        with self.tracker.await_insert(
            fixtures.FILESYSTEM_GRAPH, f"nie:url <{check_file}>", timeout=cfg.AWAIT_TIMEOUT
        ):
            os.rename(source, dest)

        # Avoid checks to ensure the indexer didn't complete

        file_id = self.tracker.get_content_resource_id(self.uri("test-monitored/dir"))
        with self.tracker.await_delete(
            fixtures.FILESYSTEM_GRAPH, file_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            os.rename(dest, source)

        result = self.__get_text_documents()
        # Default test data only
        self.assertEqual(len(result), 3)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)

        # Wait a bit and repeat the same checks
        time.sleep(3)
        result = self.__get_text_documents()
        self.assertEqual(len(result), 3)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)

class IndexedFolderTest(fixtures.TrackerMinerTest):
    """
    Tests handling of data across multiple data sources
    """

    def setUp(self):
        # We must create the test data before the miner does its
        # initial crawl, or it may miss some files due
        # https://gitlab.gnome.org/GNOME/tracker-miners/issues/79.
        monitored_files = self.create_test_data()

        try:
            # Start the miner.
            fixtures.TrackerMinerTest.setUp(self)

            for tf in monitored_files:
                url = self.uri(tf)
                self.tracker.ensure_resource(
                    fixtures.DOCUMENTS_GRAPH,
                    f"a nfo:Document ; nie:isStoredAs <{url}>",
                    timeout=cfg.AWAIT_TIMEOUT,
                )
        except Exception:
            cfg.remove_monitored_test_dir(self.workdir)
            raise

        logging.info("%s.setUp(): complete", self)

    def create_test_data(self):
        monitored_files = [
            "test-monitored/file1.txt",
            "test-monitored/dir1/file3.txt",
            "test-non-recursive/file2.txt",
        ]

        unmonitored_files = ["test-no-monitored/file0.txt"]

        for tf in monitored_files:
            testfile = self.path(tf)
            os.makedirs(os.path.dirname(testfile), exist_ok=True)
            with open(testfile, "w") as f:
                f.write(DEFAULT_TEXT)

        return monitored_files

    def __get_text_documents(self):
        return self.tracker.query(
            """
          SELECT DISTINCT ?url WHERE {
              ?u a nfo:TextDocument ;
                 nie:isStoredAs/nie:url ?url.
          }
          """
        )

    def __get_index_folder(self, filepath):
        result = self.tracker.query(
            """
          SELECT DISTINCT ?p WHERE {
              ?u a nfo:FileDataObject ;
                 nie:url \"%s\" ;
                 nie:dataSource/nie:isStoredAs ?p
          }
          """
            % (self.uri(filepath))
        )
        self.assertEqual(len(result), 1)
        return result[0][0]

    """
    Move a file between indexed folders and check the data source changed
    """
    def test_01_move_between_indexed_folders(self):
        """
        Move a file between indexed folders and check the data source changed
        """
        result = self.__get_text_documents()
        self.assertEqual(len(result), 3)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/file3.txt"), unpacked_result)
        self.assertIn(self.uri("test-non-recursive/file2.txt"), unpacked_result)

        source = os.path.join(self.workdir, "test-non-recursive", "file2.txt")
        dest = os.path.join(self.workdir, "test-monitored", "file2.txt")

        datasource1 = self.__get_index_folder(source);
        self.assertEqual(datasource1, self.uri("test-non-recursive"))

        with self.await_document_inserted(dest) as resource:
            shutil.move(source, dest)

        result = self.__get_text_documents()
        self.assertEqual(len(result), 3)
        unpacked_result = [r[0] for r in result]
        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/file2.txt"), unpacked_result)
        self.assertIn(self.uri("test-monitored/dir1/file3.txt"), unpacked_result)

        # The file was moved to test-monitored
        datasource2 = self.__get_index_folder(dest);
        self.assertEqual(datasource2, self.uri("test-monitored"))
        self.assertNotEqual(datasource1, datasource2)


if __name__ == "__main__":
    fixtures.tracker_test_main()



# Copyright (C) 2025, Red Hat Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author: Carlos Garnacho <carlosg@gnome.org>

"""
Test `localsearch reset` subcommand
"""

from typing import *
import dataclasses
import pathlib
import os
import time

import configuration
import fixtures
import shutil

from minerhelper import MinerFsHelper

class TestCli(fixtures.TrackerCommandLineTestCase):
    def test_reset(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        # Copy a file and wait for it to be indexed, in order to ensure idle state
        file = datadir.joinpath("text/mango.txt")
        target = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file)))
        with self.await_document_inserted(target):
            shutil.copy(file, self.indexed_dir)

        pid = self.sandbox.session_bus.get_connection_unix_process_id_sync(
            'org.freedesktop.LocalSearch3')

        # Reset the database
        self.run_cli(["localsearch", "reset"])

        # Ensure the process is gone
        pid_exists = True
        attempts = 0
        while pid_exists:
            if attempts == 10:
                raise RuntimeError("Took too long to stop indexer")

            try:
                time.sleep(1)
                program = os.readlink("/proc/" + str(pid) + "/exe")
                pid_exists = os.path.basename(program) == "localsearch-3"
                if pid_exists:
                    attempts += 1
            except:
                pid_exists = False

        # Re-start the indexer, check that file is reindexed
        with self.await_document_inserted(target):
            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()

    def test_reset_file(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        # Copy a file and wait for it to be indexed, in order to ensure state
        file = datadir.joinpath("text/mango.txt")
        target = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file)))
        with self.await_document_inserted(target):
            shutil.copy(file, target)

        output = self.run_cli(["localsearch", "status"])
        # State should be idle
        self.assertIn("idle", output)

        resource_id = self.tracker.get_content_resource_id(self.uri(target))
        with self.tracker.await_delete(fixtures.DOCUMENTS_GRAPH, resource_id):
            self.run_cli(["localsearch", "reset", "--file", target])

        # Trigger another change, wait for it to be processed
        file = datadir.joinpath("text/mango.txt")
        target2 = pathlib.Path(os.path.join(self.indexed_dir, 'mango2.txt'))
        with self.await_document_inserted(target2):
            shutil.copy(file, target2)

        # Both files should exist
        self.assertResourceExists(self.uri(target))
        self.assertResourceExists(self.uri(target2))

    def test_reset_file_not_indexed(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        # Copy a file and wait for it to be indexed, in order to ensure state
        file = datadir.joinpath("text/mango.txt")
        target = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file)))
        not_indexed = pathlib.Path(os.path.join(self.non_recursive_dir, 'folder', 'mango3.txt'))
        not_indexed.parent.mkdir(parents=True)
        with self.await_document_inserted(target):
            shutil.copy(file, target)
            shutil.copy(file, not_indexed)

        self.assertResourceExists(self.uri(target))
        self.assertResourceMissing(self.uri(not_indexed))

        output = self.run_cli(["localsearch", "status"])
        # State should be idle
        self.assertIn("idle", output)

        # Reset a file that should not be indexed in the first place
        target2 = pathlib.Path(os.path.join(self.indexed_dir, 'mango2.txt'))
        with self.await_document_inserted(target2):
            self.run_cli(["localsearch", "reset", "--file", not_indexed])
            shutil.copy(file, target2)

        # Non indexed file should stay not indexed
        self.assertResourceExists(self.uri(target))
        self.assertResourceExists(self.uri(target2))
        self.assertResourceMissing(self.uri(not_indexed))

    def test_reset_wrongargs(self):
        err = None
        out = None
        try:
            # Pass unknown arg
            out = self.run_cli(["localsearch", "reset", "--asdf"])
        except Exception as e:
            err = str(e)
        finally:
            self.assertIn("--asdf", err);
            self.assertIsNone(out)

if __name__ == "__main__":
    fixtures.tracker_test_main()



# Copyright (C) 2010, Nokia (ivan.frade@nokia.com)
# Copyright (C) 2019-2020, Sam Thursfield (sam@afuera.me.uk)
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA  02110-1301, USA.
#

"""Tests for Tracker writeback daemon."""


import logging
import unittest as ut

import configuration
import fixtures

import gi
from gi.repository import GLib

log = logging.getLogger(__name__)


class WritebackImagesTest(fixtures.TrackerWritebackTest):
    """
    Write in tracker store the properties witih writeback support and check
    that the new values are actually in the file
    """

    def __writeback_test(self, filename, data):
        """
        Set a value in @prop for the @filename. Then ask tracker-extractor
        for metadata and check in the results dictionary if the property is there.

        Note: given the special translation of some property-names in the dictionary
        with extracted metadata, there is an optional parameter @expectedKey
        to specify what property to check in the dictionary. If None, then
        the @prop is used.
        """

        path = self.prepare_test_image(self.datadir_path(filename))
        initial_mtime = path.stat().st_mtime

        resource = self.create_resource("nfo:Image", path, data)
        self.writeback_data(resource.serialize())

        log.debug("Waiting for change on %s", path)
        self.wait_for_file_change(path, initial_mtime)
        log.debug("Got the change")
        self.check_data(path, data)

    # JPEG test

    def test_001_jpeg_title(self):
        self.__writeback_test(
            "writeback-test-1.jpeg",
            {"nie:title": "test_title"})

    def test_002_jpeg_description(self):
        self.__writeback_test(
            "writeback-test-1.jpeg",
            {"nie:description": "test_description"})

    def test_003_jpeg_hasTag (self):
        self.__writeback_test (
            "writeback-test-1.jpeg",
            {"nao:hasTag" : {"nao:prefLabel": "test_tag"}})

    def test_004_jpeg_contributor (self):
        self.__writeback_test (
            "writeback-test-1.jpeg",
            {"nco:contributor": {"nco:fullname": "test_contributor"}})

    def test_005_jpeg_copyright (self):
        self.__writeback_test (
            "writeback-test-1.jpeg",
            {"nie:copyright": "test_copyright"})

    def test_006_jpeg_content_created (self):
        self.__writeback_test (
            "writeback-test-1.jpeg",
            {"nie:contentCreated": "2001-01-01T12:23:34Z"})

    def test_007_jpeg_heading (self):
        self.__writeback_test (
            "writeback-test-1.jpeg",
            {"nfo:heading": 42.0})

    def test_008_jpeg_location (self):
        self.__writeback_test (
            "writeback-test-1.jpeg",
            {"slo:location": { "slo:longitude": 123.0,
                               "slo:latitude": 12.0,
                               "slo:altitude": -1.0,
                               "slo:postalAddress": {"nco:locality": "test_locality",
                                                     "nco:region": "test_region",
                                                     "nco:streetAddress": "test_address",
                                                     "nco:country": "test_country"}}})

    # TIFF tests

    def test_011_tiff_title(self):
        self.__writeback_test(
            "writeback-test-2.tif",
            {"nie:title": "test_title"})

    def test_012_tiff_description(self):
        self.__writeback_test(
            "writeback-test-2.tif",
            {"nie:description": "test_description"})

    def test_013_tiff_hasTag (self):
        self.__writeback_test (
            "writeback-test-2.tif",
            {"nao:hasTag": {"nao:prefLabel": "test_tag"}})

    def test_014_tiff_contributor (self):
        self.__writeback_test (
            "writeback-test-2.tif",
            {"nco:contributor": {"nco:fullname": "test_contributor"}})

    def test_015_tiff_copyright (self):
        self.__writeback_test (
            "writeback-test-2.tif",
            {"nie:copyright": "test_copyright"})

    def test_016_tiff_content_created (self):
        self.__writeback_test (
            "writeback-test-2.tif",
            {"nie:contentCreated": "2001-01-01T12:23:34Z"})

    # PNG tests

    def test_021_png_title(self):
        self.__writeback_test(
            "writeback-test-4.png",
            {"nie:title": "test_title"})

    def test_022_png_description(self):
        self.__writeback_test(
            "writeback-test-4.png",
            {"nie:description": "test_description"})

    def test_023_png_hasTag (self):
        self.__writeback_test (
            "writeback-test-4.png",
            {"nao:hasTag": {"nao:prefLabel": "test_tag"}})

    def test_024_png_contributor (self):
        self.__writeback_test (
            "writeback-test-4.png",
            {"nco:contributor": {"nco:fullname": "test_contributor"}})

    def test_025_png_copyright (self):
        self.__writeback_test (
            "writeback-test-4.png",
            {"nie:copyright": "test_copyright"})

    def test_026_png_content_created (self):
        self.__writeback_test (
            "writeback-test-4.png",
            {"nie:contentCreated": "2001-01-01T12:23:34Z"})

    def test_027_png_heading (self):
        self.__writeback_test (
            "writeback-test-4.png",
            {"nfo:heading": 42.2})

    def test_028_png_location (self):
        self.__writeback_test (
            "writeback-test-4.png",
            {"slo:location": { "slo:longitude": 123.0,
                               "slo:latitude": 12.0,
                               "slo:altitude": -1.0,
                               "slo:postalAddress": {"nco:locality": "test_locality",
                                                     "nco:region": "test_region",
                                                     "nco:streetAddress": "test_address",
                                                     "nco:country": "test_country"}}})

if __name__ == "__main__":
    fixtures.tracker_test_main()



# Copyright (C) 2025, Red Hat Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author: Carlos Garnacho <carlosg@gnome.org>

"""
Test startup being delayed via systemd state monitoring
"""

import time
import fixtures
import dbusmock
import subprocess

class TestSystemd(fixtures.TrackerMinerTest):
    def setUp(self):
        super(TestSystemd, self).setUp()
        self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
        (self.systemd, self.systemd_obj) = self.spawn_server_template(
            'systemd', {}, system_bus=False)

    def tearDown(self):
        self.systemd.terminate()
        self.systemd.wait()
        super(TestSystemd, self).tearDown()

    def test_systemd_ready(self):
        self.systemd_obj.AddProperty(
            'org.freedesktop.systemd1.Manager', 'SystemState', 'running')

        filename = "test-monitored/file1.txt"
        path = self.path(filename)
        uri = self.uri(filename)

        start = time.time()
        # This activates the indexer
        with self.await_document_inserted(path):
            with open(path, "w") as f:
                f.write("Foo bar baz")

        elapsed = time.time() - start
        # We should ideally test for this, but cannot with slow CI runners
        #self.assertTrue(elapsed < 5)

        # The file should be indexed as usual
        self.assertResourceExists(uri)


    def test_systemd_busy_then_settle(self):
        self.systemd_obj.AddProperty(
            'org.freedesktop.systemd1.Manager', 'SystemState', 'activating')

        filename = "test-monitored/file1.txt"
        path = self.path(filename)
        uri = self.uri(filename)
        with open(path, "w") as f:
            f.write("Foo bar baz")

        time.sleep(1)
        self.assertResourceMissing(uri)

        start = time.time()
        # This activates the indexer
        with self.await_document_inserted(path):
            self.systemd_obj.Set(
                'org.freedesktop.systemd1.Manager', 'SystemState', 'running')
            self.systemd_obj.EmitSignal(
                'org.freedesktop.systemd1.Manager', 'StartupFinished',
                'uuuuuu', [0, 0, 0, 0, 0, 0])

        elapsed = time.time() - start
        # We should ideally test for this, but cannot with slow CI runners
        # self.assertTrue(elapsed < 5)

        # The file should be indexed as usual
        self.assertResourceExists(uri)

    def test_systemd_busy(self):
        self.systemd_obj.AddProperty(
            'org.freedesktop.systemd1.Manager', 'SystemState', 'activating')

        filename = "test-monitored/file1.txt"
        path = self.path(filename)
        uri = self.uri(filename)

        start = time.time()

        # This activates the indexer
        with self.await_document_inserted(path):
            with open(path, "w") as f:
                f.write("Foo bar baz")

        elapsed = time.time() - start
        self.assertTrue(elapsed >= 5)

        # The file should be indexed as usual
        self.assertResourceExists(uri)

        # Check that belated startup finished does not have ill effects
        self.systemd_obj.Set(
            'org.freedesktop.systemd1.Manager', 'SystemState', 'running')
        self.systemd_obj.EmitSignal(
            'org.freedesktop.systemd1.Manager', 'StartupFinished',
            'uuuuuu', [0, 0, 0, 0, 0, 0])
        time.sleep(1)
        self.assertResourceExists(uri)


if __name__ == "__main__":
    fixtures.tracker_test_main()



# Copyright (C) 2025, Red Hat Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author: Carlos Garnacho <carlosg@gnome.org>

"""
Test (somewhat) memory pressure handling
"""

import fixtures
import dbusmock
import subprocess

class TestMemoryPressure(fixtures.TrackerMinerTest):
    def setUp(self):
        self.start_system_bus()
        (self.low_memory_monitor, self.low_memory_monitor_obj) = self.spawn_server_template(
            'low_memory_monitor', {})
        super(TestMemoryPressure, self).setUp()

    def tearDown(self):
        self.low_memory_monitor.terminate()
        self.low_memory_monitor.wait()
        super(TestMemoryPressure, self).tearDown()

    def test_memory_pressure(self):
        self.low_memory_monitor_obj.EmitSignal(
            'org.freedesktop.LowMemoryMonitor',
            'LowMemoryWarning', 'y', [255])

        # We cannot test for the effects, test at least
        # that everything works as usual.
        filename1 = "test-monitored/file1.txt"
        path1 = self.path(filename1)
        uri1 = self.uri(filename1)

        with self.await_document_inserted(path1):
            with open(path1, "w") as f:
                f.write("Foo bar baz")

        self.assertResourceExists(uri1)
        self.low_memory_monitor_obj.EmitWarning(255)


if __name__ == "__main__":
    fixtures.tracker_test_main()



# Copyright (C) 2019-2020, Sam Thursfield (sam@afuera.me.uk)
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA  02110-1301, USA.


import unittest

import fixtures

import gi
from gi.repository import GLib


class WritebackAudioTest(fixtures.TrackerWritebackTest):
    def _writeback_test(self, path, data):
        path = self.prepare_test_audio(self.datadir_path(path))
        initial_mtime = path.stat().st_mtime

        resource = self.create_resource("nfo:Audio", path, data)
        self.writeback_data(resource.serialize())

        self.wait_for_file_change(path, initial_mtime)
        self.check_data(path, data)

    # Missing metadata:
    # nmm:artwork, nmm:internationalStandardRecordingCode,
    # nmm:lyrics, nie:description, nao:hasTag

    # MP3

    def test_mp3_title(self):
        self._writeback_test(
            "writeback-test-5.mp3",
            {"nie:title": "test_title"})

    def test_mp3_comment(self):
        self._writeback_test(
            "writeback-test-5.mp3",
            {"nie:comment": "test_comment"})

    def test_mp3_genre(self):
        self._writeback_test(
            "writeback-test-5.mp3",
            {"nfo:genre": "test_genre"})

    def test_mp3_artist(self):
        self._writeback_test(
            "writeback-test-5.mp3",
            {"nmm:artist": {"nmm:artistName": "test_artist",
                            "tracker:hasExternalReference": {"tracker:referenceSource": "https://musicbrainz.org/doc/Artist",
                                                             "tracker:referenceIdentifier": "test_mb_identifier"}}})

    @unittest.skip('gstreamer does not write the tag')
    def test_mp3_mb_track(self):
        self._writeback_test(
            "writeback-test-5.mp3",
            {"tracker:hasExternalReference": [{"tracker:referenceSource": "https://musicbrainz.org/doc/Recording",
                                               "tracker:referenceIdentifier": "test_mb_recording"},
                                              {"tracker:referenceSource": "https://musicbrainz.org/doc/Track",
                                               "tracker:referenceIdentifier": "test_mb_track"}]})

    def test_mp3_track_number(self):
        self._writeback_test(
            "writeback-test-5.mp3",
            {"nmm:trackNumber": 42})

    @unittest.skip('gstreamer does not write the tag')
    def test_mp3_content_created(self):
        self._writeback_test(
            "writeback-test-5.mp3",
            {"nie:contentCreated": "1234-12-23T00:00:00Z"})

    @unittest.skip('gstreamer does not write the tag')
    def test_mp3_isrc(self):
        self._writeback_test(
            "writeback-test-5.mp3",
            {"nmm:internationalStandardRecordingCode": "test_isrc"})

    def test_mp3_composer(self):
        self._writeback_test(
            "writeback-test-5.mp3",
            {"nmm:composer": {"nmm:artistName": "test_composer"}})

    @unittest.skip('gstreamer does not write the tag')
    def test_mp3_publisher(self):
        self._writeback_test(
            "writeback-test-5.mp3",
            {"nco:publisher": {"nco:fullname": "test_publisher"}})

    @unittest.skip('gstreamer does not write the tag')
    def test_mp3_hasHash (self):
        self._writeback_test (
            "writeback-test-5.mp3",
            {"nfo:hasHash" : {"nfo:hashAlgorithm": "chromaprint",
                              "nfo:hashValue": "test_hash"}})

    def test_mp3_music_album(self):
        self._writeback_test(
            "writeback-test-5.mp3",
            {"nmm:musicAlbum": {"nie:title": "test_title",
                                "nmm:albumArtist": { "nmm:artistName": "test_artist"},
                                "tracker:hasExternalReference": [{"tracker:referenceSource": "https://musicbrainz.org/doc/Release",
                                                                  "tracker:referenceIdentifier": "test_mb_release"},
                                                                 {"tracker:referenceSource": "https://musicbrainz.org/doc/Release_Group",
                                                                  "tracker:referenceIdentifier": "test_mb_release_group"}]},
             "nmm:musicAlbumDisc": {"nmm:setNumber": 42}})

    # Ogg

    def test_ogg_title(self):
        self._writeback_test(
            "writeback-test-6.ogg",
            {"nie:title": "test_title"})

    @unittest.skip('gstreamer does not write the tag')
    def test_ogg_comment(self):
        self._writeback_test(
            "writeback-test-6.ogg",
            {"nie:comment": "test_comment"})

    def test_ogg_genre(self):
        self._writeback_test(
            "writeback-test-6.ogg",
            {"nfo:genre": "test_genre"})

    def test_ogg_artist(self):
        self._writeback_test(
            "writeback-test-6.ogg",
            {"nmm:artist": {"nmm:artistName": "test_artist",
                            "tracker:hasExternalReference": {"tracker:referenceSource": "https://musicbrainz.org/doc/Artist",
                                                             "tracker:referenceIdentifier": "test_mb_identifier"}}})

    def test_ogg_mb_track(self):
        self._writeback_test(
            "writeback-test-6.ogg",
            {"tracker:hasExternalReference": [{"tracker:referenceSource": "https://musicbrainz.org/doc/Recording",
                                               "tracker:referenceIdentifier": "test_mb_recording"},
                                              {"tracker:referenceSource": "https://musicbrainz.org/doc/Track",
                                               "tracker:referenceIdentifier": "test_mb_track"}]})

    def test_ogg_track_number(self):
        self._writeback_test(
            "writeback-test-6.ogg",
            {"nmm:trackNumber": 42})

    def test_ogg_content_created(self):
        self._writeback_test(
            "writeback-test-6.ogg",
            {"nie:contentCreated": "1234-12-23T00:00:00Z"})

    @unittest.skip('gstreamer does not write the tag')
    def test_ogg_isrc(self):
        self._writeback_test(
            "writeback-test-6.ogg",
            {"nmm:internationalStandardRecordingCode": "test_isrc"})

    def test_ogg_composer(self):
        self._writeback_test(
            "writeback-test-6.ogg",
            {"nmm:composer": {"nmm:artistName": "test_composer"}})

    @unittest.skip('gstreamer does not write the tag')
    def test_ogg_publisher(self):
        self._writeback_test(
            "writeback-test-6.ogg",
            {"nco:publisher": {"nco:fullname": "test_publisher"}})

    def test_ogg_hasHash (self):
        self._writeback_test (
            "writeback-test-6.ogg",
            {"nfo:hasHash" : {"nfo:hashAlgorithm": "chromaprint",
                              "nfo:hashValue": "test_hash"}})

    def test_ogg_music_album(self):
        self._writeback_test(
            "writeback-test-6.ogg",
            {"nmm:musicAlbum": {"nie:title": "test_title",
                                "nmm:albumArtist": { "nmm:artistName": "test_artist"},
                                "tracker:hasExternalReference": [{"tracker:referenceSource": "https://musicbrainz.org/doc/Release",
                                                                  "tracker:referenceIdentifier": "test_mb_release"},
                                                                 {"tracker:referenceSource": "https://musicbrainz.org/doc/Release_Group",
                                                                  "tracker:referenceIdentifier": "test_mb_release_group"}]},
             "nmm:musicAlbumDisc": {"nmm:setNumber": 42}})

    # Flac

    def test_flac_title(self):
        self._writeback_test(
            "writeback-test-7.flac",
            {"nie:title": "test_title"})

    @unittest.skip('gstreamer does not write the tag')
    def test_flac_comment(self):
        self._writeback_test(
            "writeback-test-7.flac",
            {"nie:comment": "test_comment"})

    def test_flac_genre(self):
        self._writeback_test(
            "writeback-test-7.flac",
            {"nfo:genre": "test_genre"})

    def test_flac_artist(self):
        self._writeback_test(
            "writeback-test-7.flac",
            {"nmm:artist": {"nmm:artistName": "test_artist",
                            "tracker:hasExternalReference": {"tracker:referenceSource": "https://musicbrainz.org/doc/Artist",
                                                             "tracker:referenceIdentifier": "test_mb_identifier"}}})

    def test_flac_mb_track(self):
        self._writeback_test(
            "writeback-test-7.flac",
            {"tracker:hasExternalReference": [{"tracker:referenceSource": "https://musicbrainz.org/doc/Recording",
                                               "tracker:referenceIdentifier": "test_mb_recording"},
                                              {"tracker:referenceSource": "https://musicbrainz.org/doc/Track",
                                               "tracker:referenceIdentifier": "test_mb_track"}]})

    def test_flac_track_number(self):
        self._writeback_test(
            "writeback-test-7.flac",
            {"nmm:trackNumber": 42})

    def test_flac_content_created(self):
        self._writeback_test(
            "writeback-test-7.flac",
            {"nie:contentCreated": "1234-12-23T00:00:00Z"})

    @unittest.skip('gstreamer does not write the tag')
    def test_flac_isrc(self):
        self._writeback_test(
            "writeback-test-7.flac",
            {"nmm:internationalStandardRecordingCode": "test_isrc"})

    def test_flac_composer(self):
        self._writeback_test(
            "writeback-test-7.flac",
            {"nmm:composer": {"nmm:artistName": "test_composer"}})

    @unittest.skip('gstreamer does not write the tag')
    def test_flac_publisher(self):
        self._writeback_test(
            "writeback-test-7.flac",
            {"nco:publisher": {"nco:fullname": "test_publisher"}})

    def test_flac_hasHash (self):
        self._writeback_test (
            "writeback-test-7.flac",
            {"nfo:hasHash" : {"nfo:hashAlgorithm": "chromaprint",
                              "nfo:hashValue": "test_hash"}})

    def test_flac_music_album(self):
        self._writeback_test(
            "writeback-test-7.flac",
            {"nmm:musicAlbum": {"nie:title": "test_title",
                                "nmm:albumArtist": { "nmm:artistName": "test_artist"},
                                "tracker:hasExternalReference": [{"tracker:referenceSource": "https://musicbrainz.org/doc/Release",
                                                                  "tracker:referenceIdentifier": "test_mb_release"},
                                                                 {"tracker:referenceSource": "https://musicbrainz.org/doc/Release_Group",
                                                                  "tracker:referenceIdentifier": "test_mb_release_group"}]},
             "nmm:musicAlbumDisc": {"nmm:setNumber": 42}})

    # AAC/MP4

    def test_aac_title(self):
        self._writeback_test(
            "writeback-test-8.mp4",
            {"nie:title": "test_title"})

    def test_aac_comment(self):
        self._writeback_test(
            "writeback-test-8.mp4",
            {"nie:comment": "test_comment"})

    def test_aac_genre(self):
        self._writeback_test(
            "writeback-test-8.mp4",
            {"nfo:genre": "test_genre"})

    @unittest.skip('gstreamer does not write the tag')
    def test_aac_artist(self):
        self._writeback_test(
            "writeback-test-8.mp4",
            {"nmm:artist": {"nmm:artistName": "test_artist",
                            "tracker:hasExternalReference": {"tracker:referenceSource": "https://musicbrainz.org/doc/Artist",
                                                             "tracker:referenceIdentifier": "test_mb_identifier"}}})

    @unittest.skip('gstreamer does not write the tag')
    def test_aac_mb_track(self):
        self._writeback_test(
            "writeback-test-8.mp4",
            {"tracker:hasExternalReference": [{"tracker:referenceSource": "https://musicbrainz.org/doc/Recording",
                                               "tracker:referenceIdentifier": "test_mb_recording"},
                                              {"tracker:referenceSource": "https://musicbrainz.org/doc/Track",
                                               "tracker:referenceIdentifier": "test_mb_track"}]})

    def test_aac_track_number(self):
        self._writeback_test(
            "writeback-test-8.mp4",
            {"nmm:trackNumber": 42})

    def test_aac_content_created(self):
        self._writeback_test(
            "writeback-test-8.mp4",
            {"nie:contentCreated": "1234-12-23T00:00:00.000000Z"})

    @unittest.skip('gstreamer does not write the tag')
    def test_aac_isrc(self):
        self._writeback_test(
            "writeback-test-8.mp4",
            {"nie:title": "test_title",
             "nmm:internationalStandardRecordingCode": "test_isrc"})

    def test_aac_composer(self):
        self._writeback_test(
            "writeback-test-8.mp4",
            {"nmm:composer": {"nmm:artistName": "test_composer"}})

    @unittest.skip('gstreamer does not write the tag')
    def test_aac_publisher(self):
        self._writeback_test(
            "writeback-test-8.mp4",
            {"nco:publisher": {"nco:fullname": "test_publisher"}})

    @unittest.skip('gstreamer does not write the tag')
    def test_aac_hasHash (self):
        self._writeback_test (
            "writeback-test-8.mp4",
            {"nfo:hasHash" : {"nfo:hashAlgorithm": "chromaprint",
                              "nfo:hashValue": "test_hash"}})

    @unittest.skip('gstreamer does not write the tag')
    def test_aac_music_album(self):
        self._writeback_test(
            "writeback-test-8.mp4",
            {"nmm:musicAlbum": {"nie:title": "test_title",
                                "nmm:albumArtist": { "nmm:artistName": "test_artist"},
                                "tracker:hasExternalReference": [{"tracker:referenceSource": "https://musicbrainz.org/doc/Release",
                                                                  "tracker:referenceIdentifier": "test_mb_release"},
                                                                 {"tracker:referenceSource": "https://musicbrainz.org/doc/Release_Group",
                                                                  "tracker:referenceIdentifier": "test_mb_release_group"}]},
             "nmm:musicAlbumDisc": {"nmm:setNumber": 42}})


if __name__ == "__main__":
    fixtures.tracker_test_main()



# Copyright (C) 2020, Sam Thursfield <sam@afuera.me.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

"""
Test `localsearch` commandline tool
"""

from typing import *
import dataclasses
import pathlib
import os
import re

import configuration
import fixtures
import shutil


class TestCli(fixtures.TrackerCommandLineTestCase):
    def test_noargs(self):
        out = self.run_cli(["localsearch"])
        self.assertIn("Available localsearch commands are", out)

    def test_arg_help(self):
        out = self.run_cli(["localsearch", "--help"])
        self.assertIn("Available localsearch commands are", out)

    def test_wrongargs(self):
        out = ""
        err = ""
        try:
            out = self.run_cli(["localsearch", "asdf"])
        except Exception as e:
            err = str(e)

        self.assertEqual("", out)
        self.assertIn("CLI command failed", err)

    def test_arg_version(self):
        out = self.run_cli(["localsearch", "--version"])
        self.assertIn("LocalSearch 3", out)
        self.assertIn("General Public License", out)

    def test_help(self):
        output = self.run_cli(["localsearch", "help", "info"])
        self.assertIn("localsearch", output)

    def test_help_wrongargs(self):
        out = ""
        err = ""
        try:
            out = self.run_cli(["localsearch", "help", "asdf"])
        except Exception as e:
            err = str(e)

        self.assertEqual("", out)
        self.assertIn("CLI command failed", err)

if __name__ == "__main__":
    fixtures.tracker_test_main()



# Copyright (C) 2020, Sam Thursfield <sam@afuera.me.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

"""
Test `localsearch search` subcommand
"""

from typing import *
import dataclasses
import pathlib
import os
import re

import configuration
import fixtures
import shutil

class TestCli(fixtures.TrackerCommandLineTestCase):
    def test_search(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        # FIXME: synchronous `tracker index` isn't ready yet;
        # see https://gitlab.gnome.org/GNOME/tracker/-/issues/188
        # in the meantime we manually wait for it to finish.

        file1 = datadir.joinpath("text/Document 1.txt")
        target1 = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file1)))
        with self.await_document_inserted(target1):
            shutil.copy(file1, self.indexed_dir)

        file2 = datadir.joinpath("text/Document 2.txt")
        target2 = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file2)))
        with self.await_document_inserted(target2):
            shutil.copy(file2, self.indexed_dir)

        folder_name = "test-folder"
        folder_path = pathlib.Path(os.path.join(self.indexed_dir, folder_name))
        with self.await_insert_dir(folder_path):
            try:
                os.mkdir(os.path.join(self.indexed_dir, folder_name))
            except OSError as error:
                print(error)

        # FIXME: the --all should NOT be needed.
        # See: https://gitlab.gnome.org/GNOME/tracker-miners/-/issues/116
        output = self.run_cli(["localsearch", "search", "--all", "banana"])
        self.assertIn(target1.as_uri(), output)
        self.assertNotIn(target2.as_uri(), output)

        folder_output = self.run_cli(
            ["localsearch", "search", "--folders", "test-monitored"]
        )
        self.assertIn(self.indexed_dir, folder_output)
        self.assertNotIn(folder_path.as_uri(), folder_output)

    def test_search_filename(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        file1 = datadir.joinpath("text/mango.txt")
        target1 = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file1)))
        with self.await_document_inserted(target1):
            shutil.copy(file1, self.indexed_dir)

        target2 = pathlib.Path(os.path.join(self.indexed_dir, "Document 2.txt"))

        search_output = self.run_cli(["localsearch", "search", "mango"])
        self.assertIn(target1.as_uri(), search_output)
        self.assertNotIn(target2.as_uri(), search_output)

    def test_search_detailed(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        file1 = datadir.joinpath("text/mango.txt")
        target1 = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file1)))
        with self.await_document_inserted(target1):
            shutil.copy(file1, self.indexed_dir)

        # Check we get fts context
        search_output = self.run_cli(["localsearch", "search", "--detailed", "lorem"])
        self.assertIn(target1.as_uri(), search_output)
        self.assertIn("dolor", search_output)

    def test_search_noargs(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        file1 = datadir.joinpath("text/mango.txt")
        target1 = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file1)))
        with self.await_document_inserted(target1):
            shutil.copy(file1, self.indexed_dir)

        search_output = self.run_cli(["localsearch", "search"])
        self.assertIn(self.uri(self.indexed_dir), search_output)
        self.assertIn(target1.as_uri(), search_output)

    def test_search_wrongargs(self):
        err = None
        out = None
        try:
            out = self.run_cli(["localsearch", "search", "--asdf"])
        except Exception as e:
            err = str(e)
        finally:
            self.assertIn("Usage", err);
            self.assertIsNone(out)

    def test_search_help(self):
        output = self.run_cli(["localsearch", "search", "--help"])
        self.assertIn("Usage", output)

if __name__ == "__main__":
    fixtures.tracker_test_main()



# Copyright (C) 2025, Red Hat Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author: Carlos Garnacho <carlosg@gnome.org>

"""
Test offline configuration changes
"""

import gi
from gi.repository import GLib

import os
import pathlib
import time
import configuration as cfg
import fixtures
from trackertestutils.dconf import DConfClient
from fixtures import MountFlags
from minerhelper import MinerFsHelper

class TestOfflineConfigChanges(fixtures.TrackerMinerTest):
    def setUp(self):
        os.makedirs(self.non_recursive_dir, exist_ok=True)
        super(TestOfflineConfigChanges, self).setUp()


    def test_ignored_files(self):
        filename1 = "test-monitored/file1.txt"
        path1 = self.path(filename1)
        uri1 = self.uri(filename1)
        with self.await_document_inserted(path1):
            with open(path1, "w") as f:
                f.write("Foo bar baz")

        self.assertResourceExists(uri1)
        resource_id = self.tracker.get_content_resource_id(uri1)

        dconf = self.sandbox.get_dconf_client()

        filename2 = "test-monitored/file2.txt"
        path2 = self.path(filename2)
        uri2 = self.uri(filename2)

        with self.tracker.await_delete(
            fixtures.DOCUMENTS_GRAPH, resource_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'ignored-files', GLib.Variant.new_strv(['*.txt']))

            with open(path2, "w") as f:
                f.write("Foo bar baz")

            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()

        # Newly created filtered file should not appear either
        self.assertResourceMissing(uri2)

        # Ensure that both files are back by removing the filter
        with self.await_document_inserted(path1):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'ignored-files', GLib.Variant.new_strv([]))

            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()

        self.assertResourceExists(uri2)


    def test_ignored_directories(self):
        filename = "test-monitored/dir/file1.txt"
        f = pathlib.Path(self.path(filename))
        uri = self.uri(filename)
        dir_uri = self.uri("test-monitored/dir")
        with self.await_document_inserted(f):
            f.parent.mkdir(parents=True, exist_ok=True)
            f.write_text("Foo bar baz")

        self.assertResourceExists(uri)
        resource_id = self.tracker.get_content_resource_id(dir_uri)

        dconf = self.sandbox.get_dconf_client()

        # Set ignored-directories filter, directory and file should disappear
        with self.tracker.await_delete(
            fixtures.FILESYSTEM_GRAPH, resource_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'ignored-directories', GLib.Variant.new_strv(['dir']))

            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()

        self.assertResourceMissing(dir_uri)
        self.assertResourceMissing(uri)

        # Ensure the file/dir are back after removing the filter
        with self.await_document_inserted(f):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'ignored-directories', GLib.Variant.new_strv([]))

            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()

        self.assertResourceExists(dir_uri)
        self.assertResourceExists(uri)


    def test_ignored_directories_with_content(self):
        filename = "test-monitored/dir/foo.txt"
        f = pathlib.Path(self.path(filename))
        f.parent.mkdir(parents=True, exist_ok=True)
        uri = self.uri(filename)
        dir_uri = self.uri("test-monitored/dir")
        hidden = pathlib.Path(self.path("test-monitored/dir/hide-me"))
        hidden.write_text("")
        with self.await_document_inserted(f):
            f.write_text("Foo bar baz")

        self.assertResourceExists(uri)
        resource_id = self.tracker.get_content_resource_id(uri)

        dconf = self.sandbox.get_dconf_client()

        # Set ignored-directories-with-content filter, dir and file should disappear
        with self.tracker.await_delete(
            fixtures.DOCUMENTS_GRAPH, resource_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'ignored-directories-with-content', GLib.Variant.new_strv(['hide-me']))

            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()

        self.assertResourceMissing(dir_uri)
        self.assertResourceMissing(uri)

        # Ensure the file/dir are back after removing the filter
        with self.await_document_inserted(f):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'ignored-directories-with-content', GLib.Variant.new_strv([]))

            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()

        self.assertResourceExists(dir_uri)
        self.assertResourceExists(uri)


    def test_index_single_directories(self):
        filename = "test-non-recursive/file1.txt"
        path = self.path(filename)
        uri = self.uri(filename)
        dir_uri = self.uri("test-non-recursive")
        with self.await_document_inserted(path):
            with open(path, "w") as f:
                f.write("Foo bar baz")

        self.assertResourceExists(uri)
        self.assertResourceExists(dir_uri)
        resource_id = self.tracker.get_content_resource_id(dir_uri)

        dconf = self.sandbox.get_dconf_client()

        with self.tracker.await_delete(
            fixtures.DOCUMENTS_GRAPH, resource_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'index-single-directories', GLib.Variant.new_strv([]))
            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()

        self.assertResourceMissing(dir_uri)
        self.assertResourceMissing(uri)

        with self.await_document_inserted(path):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'index-single-directories', GLib.Variant.new_strv([self.non_recursive_dir]))

            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()

        self.assertResourceExists(dir_uri)
        self.assertResourceExists(uri)


    def test_index_recursive_directories(self):
        filename = "test-monitored/file1.txt"
        path = self.path(filename)
        uri = self.uri(filename)
        dir_uri = self.uri("test-monitored")
        with self.await_document_inserted(path):
            with open(path, "w") as f:
                f.write("Foo bar baz")

        self.assertResourceExists(uri)
        self.assertResourceExists(dir_uri)
        resource_id = self.tracker.get_content_resource_id(dir_uri)

        dconf = self.sandbox.get_dconf_client()

        # Remove directory from configuration, file should disappear
        with self.tracker.await_delete(
            fixtures.DOCUMENTS_GRAPH, resource_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'index-recursive-directories', GLib.Variant.new_strv([]))

            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()

        self.assertResourceMissing(dir_uri)
        self.assertResourceMissing(uri)

        # Ensure that dir and file are back after configuring it again
        with self.await_document_inserted(path):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'index-recursive-directories', GLib.Variant.new_strv([self.indexed_dir]))

            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()

        self.assertResourceExists(dir_uri)
        self.assertResourceExists(uri)


    def test_enable_monitors(self):
        dconf = self.sandbox.get_dconf_client()

        start_check = "test-monitored/start_check.txt"

        # Restart the indexer
        with self.await_document_inserted(self.path(start_check)):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'enable-monitors', GLib.Variant.new_boolean(False))

            with open(self.path(start_check), "w") as f:
                f.write("Foo bar baz")

            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()

        # Create file, test that the file is not detected
        filename1 = "test-monitored/file1.txt"
        path1 = self.path(filename1)
        uri1 = self.uri(filename1)
        with open(path1, "w") as f:
            f.write("Foo bar baz")

        time.sleep(2)
        self.assertResourceMissing(uri1)


class TestMakeHomedirRecursive(fixtures.TrackerMinerTest):
    def setUp(self):
        self.nested_recursive_dir = os.path.join(self.non_recursive_dir, 'recursive')
        os.makedirs(self.nested_recursive_dir, exist_ok=True)
        self.folder_in_nested_recursive_dir = os.path.join(self.nested_recursive_dir, 'folder1')
        os.makedirs(self.folder_in_nested_recursive_dir, exist_ok=True)
        self.nested_non_recursive_dir = os.path.join(self.non_recursive_dir, 'non-recursive')
        os.makedirs(self.nested_non_recursive_dir, exist_ok=True)
        self.folder_in_nested_non_recursive_dir = os.path.join(self.nested_non_recursive_dir, 'folder2')
        os.makedirs(self.folder_in_nested_non_recursive_dir, exist_ok=True)
        super(TestMakeHomedirRecursive, self).setUp()


    def config(self):
        settings = {
            "org.freedesktop.Tracker3.Miner.Files": {
                "index-recursive-directories": GLib.Variant.new_strv(
                    [self.nested_recursive_dir]
                ),
                "index-single-directories": GLib.Variant.new_strv(
                    [self.non_recursive_dir, self.nested_non_recursive_dir]
                ),
            }
        }
        return settings


    def __get_index_folder(self, filepath):
        result = self.tracker.query(
            """
          SELECT DISTINCT ?p WHERE {
              ?u a nfo:FileDataObject ;
                 nie:url \"%s\" ;
                 nie:dataSource/nie:isStoredAs ?p
          }
          """
            % (self.uri(filepath))
        )
        self.assertEqual(len(result), 1)
        return result[0][0]


    def __list_index_folders(self):
        return self.tracker.query(
            """
            SELECT DISTINCT ?u WHERE {
              ?root a tracker:IndexedFolder;
                nie:isStoredAs ?u .
            }
            """
        )


    def test_make_homedir_recursive(self):
        filename = "test-non-recursive/file1.txt"
        rec_filename = "test-non-recursive/recursive/folder1/file2.txt"
        rec_filename2 = "test-non-recursive/non-recursive/folder2/file3.txt"
        non_rec_filename = "test-non-recursive/non-recursive/file4.txt"

        with open(self.path(rec_filename2), "w") as f:
            f.write(rec_filename2)

        with self.await_document_inserted(self.path(filename)):
            with open(self.path(filename), "w") as f:
                f.write(filename)

        self.assertEqual(
            self.__get_index_folder(self.path(filename)),
            self.uri('test-non-recursive'))

        with self.await_document_inserted(self.path(rec_filename)):
            with open(self.path(rec_filename), "w") as f:
                f.write(rec_filename)

        self.assertEqual(
            self.__get_index_folder(self.path(rec_filename)),
            self.uri('test-non-recursive/recursive'))

        with self.await_document_inserted(self.path(non_rec_filename)):
            with open(self.path(non_rec_filename), "w") as f:
                f.write(non_rec_filename)

        self.assertEqual(
            self.__get_index_folder(self.path(non_rec_filename)),
            self.uri('test-non-recursive/non-recursive'))

        # This file should be initially not indexed
        self.assertResourceMissing(self.uri(rec_filename2))

        roots = self.__list_index_folders()
        self.assertEqual(len(roots), 3)

        # Make the base dir the only folder indexed, recursively
        dconf = self.sandbox.get_dconf_client()

        # The file that was not indexed previously, should be now
        with self.await_document_inserted(self.path(rec_filename2)):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'index-recursive-directories', GLib.Variant.new_strv([self.non_recursive_dir]))

            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'index-single-directories', GLib.Variant.new_strv([]))
            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()

        self.ensure_document_inserted(self.path(filename))
        self.ensure_document_inserted(self.path(rec_filename))
        self.ensure_document_inserted(self.path(rec_filename2))
        self.ensure_document_inserted(self.path(non_rec_filename))

        self.assertEqual(
            self.__get_index_folder(self.path(filename)),
            self.uri('test-non-recursive'))
        self.assertEqual(
            self.__get_index_folder(self.path(rec_filename)),
            self.uri('test-non-recursive'))
        self.assertEqual(
            self.__get_index_folder(self.path(rec_filename2)),
            self.uri('test-non-recursive'))
        self.assertEqual(
            self.__get_index_folder(self.path(non_rec_filename)),
            self.uri('test-non-recursive'))

        roots = self.__list_index_folders()
        self.assertEqual(roots, [[self.uri(self.non_recursive_dir)]])
        self.assertEqual(len(roots), 1)


class TestOfflineConfigMount(fixtures.TrackerMinerRemovableMediaTest):
    """Test configuration with (non-)removable mounts"""

    def config(self):
        settings = super(TestOfflineConfigMount, self).config()
        settings["org.freedesktop.Tracker3.Miner.Files"][
            "index-removable-devices"
        ] = GLib.Variant.new_boolean(False)
        return settings

    def setUp(self):
        super(TestOfflineConfigMount, self).setUp()
        self.device_path = pathlib.Path(self.workdir).joinpath("mount-1")


    def test_non_removable_in_index_single_directories(self):
        self.device_path.mkdir()
        path = self.device_path.joinpath("file1.txt")
        path.write_text("Foo bar baz")
        self.add_removable_device(self.device_path, MountFlags.NON_REMOVABLE)

        dconf = self.sandbox.get_dconf_client()

        with self.await_document_inserted(path):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'index-single-directories', GLib.Variant.new_strv([str(self.device_path)]))
            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()

        self.assertResourceExists(path.as_uri())
        resource_id = self.tracker.get_content_resource_id(path.as_uri())

        with self.tracker.await_delete(
            fixtures.DOCUMENTS_GRAPH, resource_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'index-single-directories', GLib.Variant.new_strv([]))
            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()

        self.assertResourceMissing(path.as_uri())


    def test_preconfigured_non_removable_in_index_single_directories(self):
        dconf = self.sandbox.get_dconf_client()

        with self.await_insert_dir(self.device_path):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'index-single-directories', GLib.Variant.new_strv([str(self.device_path)]))

            self.device_path.mkdir()
            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()
            self.add_removable_device(self.device_path, MountFlags.NON_REMOVABLE)

        self.assertResourceExists(self.device_path.as_uri())
        resource_id = self.tracker.get_resource_id_by_uri(self.device_path.as_uri())

        with self.tracker.await_delete(
            fixtures.FILESYSTEM_GRAPH, resource_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')

            # The new miner instance will be already unaware of the "mount"
            self.device_path.rmdir()
            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()

        self.assertResourceMissing(self.device_path.as_uri())


    def test_non_removable_in_index_recursive_directories(self):
        path = self.device_path.joinpath("dir/file1.txt")
        path.parent.mkdir(parents=True)
        path.write_text("Foo bar baz")
        self.add_removable_device(self.device_path, MountFlags.NON_REMOVABLE)

        dconf = self.sandbox.get_dconf_client()

        with self.await_document_inserted(path):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'index-recursive-directories', GLib.Variant.new_strv([self.indexed_dir, str(self.device_path)]))
            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()

        self.assertResourceExists(path.as_uri())
        resource_id = self.tracker.get_content_resource_id(path.as_uri())

        with self.tracker.await_delete(
            fixtures.DOCUMENTS_GRAPH, resource_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'index-recursive-directories', GLib.Variant.new_strv([self.indexed_dir]))
            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()


    def test_preconfigured_non_removable_in_index_recursive_directories(self):
        dconf = self.sandbox.get_dconf_client()

        with self.await_insert_dir(self.device_path):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'index-recursive-directories',
                GLib.Variant.new_strv([self.indexed_dir, str(self.device_path)]))

            self.device_path.mkdir()
            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()
            self.add_removable_device(self.device_path, MountFlags.NON_REMOVABLE)

        self.assertResourceExists(self.device_path.as_uri())
        resource_id = self.tracker.get_content_resource_id(self.device_path.as_uri())

        with self.tracker.await_delete(
            fixtures.FILESYSTEM_GRAPH, resource_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')
            self.device_path.rmdir()

            # The new miner instance will be already unaware of the "mount"
            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()


if __name__ == "__main__":
    fixtures.tracker_test_main()



# Copyright (C) 2025, Red Hat Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author: Carlos Garnacho <carlosg@gnome.org>

"""
Test `localsearch index` subcommand
"""

import gi
from gi.repository import GLib

from typing import *
import dataclasses
import pathlib
import os

import configuration
import fixtures
import shutil

from trackertestutils.dconf import DConfClient
import configuration as cfg

class TestCli(fixtures.TrackerCommandLineTestCase):
    def environment(self):
        extra_env = super(TestCli, self).environment()
        extra_env["HOME"] = self.indexed_dir
        return extra_env

    def test_index(self):
        output = self.run_cli(["localsearch", "index"])
        self.assertIn("test-monitored", output)
        self.assertIn("test-non-recursive", output)

    def test_index_help(self):
        output = self.run_cli(["localsearch", "index", "--help"])
        self.assertIn("Usage", output)

    def test_index_remove_add(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        self.tracker.ensure_resource(
            fixtures.FILESYSTEM_GRAPH,
            f"nie:isStoredAs <{self.uri(self.indexed_dir)}>",
            timeout=cfg.AWAIT_TIMEOUT,
        )
        resource_id = self.tracker.get_content_resource_id(self.uri(self.indexed_dir))

        with self.tracker.await_delete(
            fixtures.FILESYSTEM_GRAPH, resource_id, timeout=cfg.AWAIT_TIMEOUT
        ):
            self.run_cli(["localsearch", "index", "--remove", self.indexed_dir])
            output = self.run_cli(["localsearch", "index"])
            self.assertNotIn("test-monitored", output)

        file = datadir.joinpath("text/mango.txt")
        target = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file)))
        shutil.copy(file, self.indexed_dir)

        # Add the directory back, and wait for the new file to be indexed
        with self.await_document_inserted(target):
            self.run_cli(["localsearch", "index", "--recursive", "--add", self.indexed_dir])

    def test_index_alias(self):
        dconf = self.sandbox.get_dconf_client()
        dconf.write (
            'org.freedesktop.Tracker3.Miner.Files',
            'index-recursive-directories', GLib.Variant.new_strv([]))
        dconf.write (
            'org.freedesktop.Tracker3.Miner.Files',
            'index-single-directories', GLib.Variant.new_strv(['&DESKTOP']))
        output1 = self.run_cli(["localsearch", "index"])
        desktop_path = self.path("test-monitored/Desktop")
        self.assertIn("/Desktop", output1)

        # Ensure that things stay the same if adding the path explicitly
        self.run_cli(["localsearch", "index", "--add", desktop_path])
        output2 = self.run_cli(["localsearch", "index"])
        self.assertEqual(output1, output2)

        # Ensure that removing by path works
        self.run_cli(["localsearch", "index", "--remove", desktop_path])
        output3 = self.run_cli(["localsearch", "index"])
        self.assertNotIn("/Desktop", output3)

    def test_index_envvar(self):
        dconf = self.sandbox.get_dconf_client()
        dconf.write (
            'org.freedesktop.Tracker3.Miner.Files',
            'index-recursive-directories', GLib.Variant.new_strv([]))
        dconf.write (
            'org.freedesktop.Tracker3.Miner.Files',
            'index-single-directories', GLib.Variant.new_strv(['$HOME']))
        output1 = self.run_cli(["localsearch", "index"])
        self.assertIn("test-monitored", output1)

        # Ensure that things stay the same if adding the path explicitly
        self.run_cli(["localsearch", "index", "--add", self.indexed_dir])
        output2 = self.run_cli(["localsearch", "index"])
        self.assertEqual(output1, output2)

        # Ensure that removing by path works
        self.run_cli(["localsearch", "index", "--remove", self.indexed_dir])
        output3 = self.run_cli(["localsearch", "index"])
        self.assertNotIn("test-monitored", output3)

    def test_index_idempotent_add(self):
        output1 = self.run_cli(["localsearch", "index"])
        self.assertIn("test-monitored", output1)
        self.run_cli(["localsearch", "index", "--recursive", "--add", self.indexed_dir])
        output2 = self.run_cli(["localsearch", "index"])
        self.assertEqual(output1, output2)

    def test_index_idempotent_remove(self):
        output1 = self.run_cli(["localsearch", "index"])
        self.assertIn("test-monitored", output1)
        # Remove non indexed path
        self.run_cli(["localsearch", "index", "--remove", "/abc"])
        output2 = self.run_cli(["localsearch", "index"])
        self.assertEqual(output1, output2)

    def test_index_wrongargs(self):
        err = None
        out = None
        try:
            # Pass unknown arg
            out = self.run_cli(["localsearch", "index", "--asdf"])
        except Exception as e:
            err = str(e)
        finally:
            self.assertIn("--asdf", err);
            self.assertIsNone(out)

    def test_index_wrongargs2(self):
        err = None
        out = None
        try:
            # Pass pass both --add and --remove
            out = self.run_cli(["localsearch", "index", "--add", "--remove", self.indexed_dir])
        except Exception as e:
            err = str(e)
        finally:
            self.assertIn("mutually exclusive", err);
            self.assertIsNone(out)

    def test_index_add_noargs(self):
        err = None
        out = None
        try:
            # Pass bare --add with no arguments
            out = self.run_cli(["localsearch", "index", "--add"])
        except Exception as e:
            err = str(e)
        finally:
            self.assertIn("Please specify", err);
            self.assertIsNone(out)

    def test_index_remove_noargs(self):
        err = None
        out = None
        try:
            # Pass bare --remove with no arguments
            out = self.run_cli(["localsearch", "index", "--remove"])
        except Exception as e:
            err = str(e)
        finally:
            self.assertIn("Please specify", err);
            self.assertIsNone(out)

    def test_index_recursive_noargs(self):
        err = None
        out = None
        try:
            # Pass bare --recursive with no more arguments
            out = self.run_cli(["localsearch", "index", "--recursive"])
        except Exception as e:
            err = str(e)
        finally:
            self.assertIn("Please specify", err);
            self.assertIsNone(out)

    def test_index_recursive_wrongargs(self):
        err = None
        out = None
        try:
            # Pass bare --recursive with no more specifiers
            out = self.run_cli(["localsearch", "index", "--recursive", "/tmp"])
        except Exception as e:
            err = str(e)
        finally:
            self.assertIn("Either --add or --remove", err);
            self.assertIsNone(out)

    def test_index_recursive_wrongargs2(self):
        err = None
        out = None
        try:
            # Pass --recursive with --remove
            out = self.run_cli(["localsearch", "index", "--recursive", "--remove", self.indexed_dir])
        except Exception as e:
            err = str(e)
        finally:
            self.assertIn("requires --add", err);
            self.assertIsNone(out)

if __name__ == "__main__":
    fixtures.tracker_test_main()



# Copyright (C) 2021, Codethink Ltd
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA  02110-1301, USA.
#
# Author: Sam Thursfield <sam@afuera.me.uk>


import logging
import pathlib

import fixtures
from fixtures import UnmountFlags

from gi.repository import GLib

import os
import stat
import time
from minerhelper import MinerFsHelper

log = logging.getLogger(__name__)


class MinerRemovableMediaTest(fixtures.TrackerMinerRemovableMediaTest):
    def setUp(self):
        super(MinerRemovableMediaTest, self).setUp()

        self.device_path = pathlib.Path(self.workdir).joinpath("removable-device-1")
        self.device_path.mkdir()

    def ensure_document_in_removable(self, endpoint_helper, uri):
        return endpoint_helper.ensure_resource(
            fixtures.DOCUMENTS_GRAPH, f"nie:isStoredAs <{uri}>",
        )

    def assertResourceExists(self, helper, urn):
        if helper.ask("ASK { <%s> a rdfs:Resource }" % urn) == False:
            self.fail(f"Resource <{urn}> does not exist")

    def assertResourceMissing(self, helper, urn):
        if helper.ask("ASK { <%s> a rdfs:Resource }" % urn) == True:
            self.fail(f"Resource <{urn}> should not exist")

    def create_test_data(self):
        files = ["file1.txt", "dir1/file2.txt"]

        for f in files:
            path = self.device_path.joinpath(f)
            path.parent.mkdir(parents=True, exist_ok=True)
            path.write_text("This file exists.")

        return files


class MinerRemovableTest(MinerRemovableMediaTest):
    """Tests for localsearch with index-removable-devices feature."""

    def test_add_remove_device(self):
        """Device should be indexed by Tracker when connected."""

        files = self.create_test_data()

        object_path = self.miner_fs.removable_device_object_path (self.device_path)
        self.add_removable_device(self.device_path)
        self.miner_fs.await_endpoint_added(self.device_path.as_uri())
        endpoint_helper = self.helper_for_endpoint(object_path)

        for f in files:
            rel_uri = self.get_relative_uri(f)
            self.ensure_document_in_removable(endpoint_helper, rel_uri)
            self.assertResourceExists(endpoint_helper, rel_uri)

        with self.miner_fs.await_endpoint_removed(self.device_path.as_uri()):
            self.remove_removable_device(self.device_path)

        endpoint_helper = None
        try:
            endpoint_helper = self.helper_for_endpoint(object_path)
        except:
            pass

        assert not endpoint_helper

    def test_remove_busy_device(self):
        """Check that a busy device becomes indexed again."""

        files = self.create_test_data()

        with self.miner_fs.await_endpoint_added(self.device_path.as_uri()):
            self.add_removable_device(self.device_path)

        object_path = self.miner_fs.removable_device_object_path(self.device_path)
        endpoint_helper = self.helper_for_endpoint(object_path)

        for f in files:
            rel_uri = self.get_relative_uri(f)
            self.ensure_document_in_removable(endpoint_helper, rel_uri)
            self.assertResourceExists(endpoint_helper, rel_uri)

        with self.miner_fs.await_endpoint_removed(self.device_path.as_uri()):
            self.remove_removable_device(self.device_path, UnmountFlags.EMULATE_BUSY)

        uri = self.device_path.as_uri();
        result = None

        try:
            result = endpoint_helper.query('SELECT (1 AS ?u) {}')
        except Exception:
            pass

        self.assertIsNone(result)

        with self.miner_fs.await_endpoint_added(self.device_path.as_uri()):
            foo = 0

        for f in files:
            rel_uri = self.get_relative_uri(f)
            self.ensure_document_in_removable(endpoint_helper, rel_uri)
            self.assertResourceExists(endpoint_helper, rel_uri)


    def test_readonly(self):
        """Device should not be indexed by Tracker when connected."""

        files = self.create_test_data()

        # Mangle database location
        dbpath = self.device_path.joinpath('.localsearch3')
        dbpath.write_text("I'm not a database")

        object_path = self.miner_fs.removable_device_object_path (self.device_path)
        self.add_removable_device(self.device_path)

        time.sleep(2)

        endpoint_helper = None
        try:
            endpoint_helper = self.helper_for_endpoint(object_path)
        except:
            pass

        assert not endpoint_helper


class MinerRemovableConfigTest(MinerRemovableMediaTest):
    def setUp(self):
        super(MinerRemovableConfigTest, self).setUp()
        self.offline_test = True

    def maybeStopService(self):
        if self.offline_test:
            self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')

    def maybeStartService(self):
        if self.offline_test:
            self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
            self.miner_fs.start()
            self.sandbox.session_bus.await_bus_name(self.MOCK_VOLUME_MONITOR_DBUS_NAME)
            self.add_removable_device(self.device_path)

    def test_ignored_files(self):
        files = self.create_test_data()

        object_path = self.miner_fs.removable_device_object_path (self.device_path)
        self.add_removable_device(self.device_path)
        self.miner_fs.await_endpoint_added(self.device_path.as_uri())
        endpoint_helper = self.helper_for_endpoint(object_path)

        self.ensure_document_in_removable(
            endpoint_helper, self.get_relative_uri('file1.txt'))

        filename1 = "file3.txt"
        rel_uri1 = self.get_relative_uri(filename1);
        path1 = self.device_path.joinpath(filename1)
        uri1 = self.uri(filename1)

        with endpoint_helper.await_insert(
            fixtures.DOCUMENTS_GRAPH, f"nie:isStoredAs <{rel_uri1}>"
        ):
            with open(path1, "w") as f:
                f.write("Foo bar baz")

        self.assertResourceExists(endpoint_helper, rel_uri1)
        resource_id = endpoint_helper.get_content_resource_id(rel_uri1)

        dconf = self.sandbox.get_dconf_client()

        # Set ignored-files filter, file should disappear
        with endpoint_helper.await_delete(fixtures.DOCUMENTS_GRAPH, resource_id):
            self.maybeStopService();
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'ignored-files', GLib.Variant.new_strv(['*.txt']))
            self.maybeStartService();

        # Newly created filtered file should not appear either
        filename2 = "file4.txt"
        rel_uri2 = self.get_relative_uri (filename2)
        path2 = self.device_path.joinpath(filename2)
        uri2 = self.uri(filename2)
        with open(path2, "w") as f:
            f.write("Foo bar baz")

        time.sleep(2)
        self.assertResourceMissing(endpoint_helper, rel_uri2)

        # Ensure that both files are back by removing the filter
        with endpoint_helper.await_insert(
            fixtures.DOCUMENTS_GRAPH, f"nie:isStoredAs <{rel_uri1}>"
        ):
            self.maybeStopService();
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'ignored-files', GLib.Variant.new_strv([]))
            self.maybeStartService();

        self.ensure_document_in_removable(endpoint_helper, rel_uri2)

    def test_ignored_directories(self):
        object_path = self.miner_fs.removable_device_object_path (self.device_path)
        self.add_removable_device(self.device_path)
        self.miner_fs.await_endpoint_added(self.device_path.as_uri())
        endpoint_helper = self.helper_for_endpoint(object_path)

        dirname = "dir"
        dir_rel_uri = self.get_relative_uri(dirname);

        filename = "dir/file3.txt"
        rel_uri = self.get_relative_uri(filename);
        path = self.device_path.joinpath(filename)

        with endpoint_helper.await_insert(
            fixtures.DOCUMENTS_GRAPH, f"nie:isStoredAs <{rel_uri}>"
        ):
            path.parent.mkdir(parents=True, exist_ok=True)
            path.write_text("Foo bar baz")

        self.assertResourceExists(endpoint_helper, dir_rel_uri)
        resource_id = endpoint_helper.get_content_resource_id(dir_rel_uri)

        dconf = self.sandbox.get_dconf_client()

        # Set ignored-directories filter, directory and file should disappear
        with endpoint_helper.await_delete(fixtures.FILESYSTEM_GRAPH, resource_id):
            self.maybeStopService();
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'ignored-directories', GLib.Variant.new_strv(['dir']))
            self.maybeStartService();

        self.assertResourceMissing(endpoint_helper, dir_rel_uri)
        self.assertResourceMissing(endpoint_helper, rel_uri)

        # Ensure the file/dir are back after removing the filter
        with endpoint_helper.await_insert(
            fixtures.DOCUMENTS_GRAPH, f"nie:isStoredAs <{rel_uri}>"
        ):
            self.maybeStopService();
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'ignored-directories', GLib.Variant.new_strv([]))
            self.maybeStartService();

        self.assertResourceExists(endpoint_helper, dir_rel_uri)
        self.assertResourceExists(endpoint_helper, rel_uri)

    def test_ignored_directories_with_content(self):
        object_path = self.miner_fs.removable_device_object_path (self.device_path)
        self.add_removable_device(self.device_path)
        self.miner_fs.await_endpoint_added(self.device_path.as_uri())
        endpoint_helper = self.helper_for_endpoint(object_path)

        dirname = "dir"
        dir_rel_uri = self.get_relative_uri(dirname);

        filename = "dir/file3.txt"
        rel_uri = self.get_relative_uri(filename);
        path = self.device_path.joinpath(filename)

        hidden = self.device_path.joinpath("dir/hide-me")

        with endpoint_helper.await_insert(
            fixtures.DOCUMENTS_GRAPH, f"nie:isStoredAs <{rel_uri}>"
        ):
            path.parent.mkdir(parents=True, exist_ok=True)
            path.write_text("Foo bar baz")
            hidden.write_text("")

        self.assertResourceExists(endpoint_helper, dir_rel_uri)
        self.assertResourceExists(endpoint_helper, rel_uri)
        resource_id = endpoint_helper.get_content_resource_id(rel_uri)

        dconf = self.sandbox.get_dconf_client()

        # Set ignored-directories-with-content filter, dir and file should disappear
        with endpoint_helper.await_delete(fixtures.DOCUMENTS_GRAPH, resource_id):
            self.maybeStopService();
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'ignored-directories-with-content', GLib.Variant.new_strv(['hide-me']))
            self.maybeStartService();

        self.assertResourceMissing(endpoint_helper, dir_rel_uri)
        self.assertResourceMissing(endpoint_helper, rel_uri)

        # Ensure the file/dir are back after removing the filter
        with endpoint_helper.await_insert(
            fixtures.DOCUMENTS_GRAPH, f"nie:isStoredAs <{rel_uri}>"
        ):
            self.maybeStopService();
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'ignored-directories-with-content', GLib.Variant.new_strv([]))
            self.maybeStartService();

        self.assertResourceExists(endpoint_helper, dir_rel_uri)
        self.assertResourceExists(endpoint_helper, rel_uri)


class MinerRemovableLiveConfigChangeTest(MinerRemovableConfigTest):
    """Tests for config changes affecting removable devices."""
    def setUp(self):
        super(MinerRemovableLiveConfigChangeTest, self).setUp()
        self.offline_test = False

    def test_index_removable_devices(self):
        self.sandbox.stop_daemon('org.freedesktop.LocalSearch3')

        dconf = self.sandbox.get_dconf_client()
        dconf.write (
            'org.freedesktop.Tracker3.Miner.Files',
            'index-removable-devices', GLib.Variant.new_boolean(False))

        self.miner_fs = MinerFsHelper(self.sandbox.get_session_bus_connection())
        self.miner_fs.start()
        self.sandbox.session_bus.await_bus_name(self.MOCK_VOLUME_MONITOR_DBUS_NAME)
        self.add_removable_device(self.device_path)

        # Ensure the endpoint is not added
        time.sleep(2)

        object_path = self.miner_fs.removable_device_object_path (self.device_path)
        endpoint_helper = None

        try:
            endpoint_helper = self.helper_for_endpoint(object_path)
        except:
            pass

        assert not endpoint_helper

        # Mountpoint should be indexed with the config change
        with self.miner_fs.await_endpoint_added(self.device_path.as_uri()):
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'index-removable-devices', GLib.Variant.new_boolean(True))

        self.miner_fs.await_endpoint_added(self.device_path.as_uri())
        endpoint_helper = self.helper_for_endpoint(object_path)
        endpoint_helper.ensure_resource(
            fixtures.FILESYSTEM_GRAPH, "nie:isStoredAs <file:>")

        # The "device" should be "removed", with the config change
        with self.miner_fs.await_endpoint_removed(self.device_path.as_uri()):
            dconf.write (
                'org.freedesktop.Tracker3.Miner.Files',
                'index-removable-devices', GLib.Variant.new_boolean(False))


if __name__ == "__main__":
    fixtures.tracker_test_main()



# Copyright (C) 2025, Red Hat Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author: Carlos Garnacho <carlosg@gnome.org>

"""
Test `localsearch status` subcommand
"""

from typing import *
import dataclasses
import pathlib
import subprocess
import os

import configuration
import fixtures
import shutil
import time

class TestCli(fixtures.TrackerCommandLineTestCase):
    def test_status(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        # Copy a file and wait for it to be indexed, in order to ensure idle state
        file = datadir.joinpath("text/mango.txt")
        target = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file)))
        with self.await_document_inserted(target):
            shutil.copy(file, self.indexed_dir)

        output = self.run_cli(["localsearch", "status"])
        self.assertIn("1 file", output)
        self.assertIn("1 folder", output)
        self.assertIn("idle", output)

    def test_status_error(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/extractor-content")

        # Copy a photo, once as a PNG with JPEG extension, known to not be extracted
        file = datadir.joinpath("images/png-photo-1.png")
        target = pathlib.Path(os.path.join(self.indexed_dir, "png-photo-1.png"))
        target2 = pathlib.Path(os.path.join(self.indexed_dir, "png-photo-1.jpeg"))

        with self.await_photo_inserted(target):
            shutil.copy(file, target2)
            shutil.copy(file, target)

        # Check for the file with wrong extension
        output = self.run_cli(["localsearch", "status"])
        self.assertIn("png-photo-1.jpeg", output)
        self.assertIn("recorded failure", output)

    def test_status_term_noerror(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        # Copy a file and wait for it to be indexed, in order to ensure idle state
        file = datadir.joinpath("text/mango.txt")
        target = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file)))
        with self.await_document_inserted(target):
            shutil.copy(file, self.indexed_dir)

        output = self.run_cli(["localsearch", "status", "mango"])
        self.assertIn("No reports found", output)

    def test_status_term_error(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/extractor-content")

        # Copy a photo, once as a PNG with JPEG extension, known to not be extracted
        file = datadir.joinpath("images/png-photo-1.png")
        target = pathlib.Path(os.path.join(self.indexed_dir, "png-photo-1.png"))
        bad_copy = pathlib.Path(os.path.join(self.indexed_dir, "png-photo-1.jpeg"))
        bad_copy2 = pathlib.Path(os.path.join(self.indexed_dir, "png-photo-2.jpeg"))

        with self.await_photo_inserted(target):
            shutil.copy(file, bad_copy)
            shutil.copy(file, bad_copy2)
            shutil.copy(file, target)

        # Check for a file with wrong extension
        output = self.run_cli(["localsearch", "status", "photo-1"])
        self.assertIn("png-photo-1.jpeg", output)
        self.assertIn("URI:", output)
        self.assertIn("Message:", output)

        # Check that the no args call lists the errors
        output = self.run_cli(["localsearch", "status"])
        self.assertIn("png-photo-1.jpeg", output)
        self.assertIn("png-photo-2.jpeg", output)

        # Check that deleting a file no longer lists its error
        os.remove(bad_copy2)
        output = self.run_cli(["localsearch", "status"])
        self.assertIn("png-photo-1.jpeg", output)
        self.assertNotIn("png-photo-2.jpeg", output)

    def test_status_stat(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        # Copy a file and wait for it to be indexed, in order to ensure state
        file = datadir.joinpath("text/mango.txt")
        target = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file)))
        with self.await_document_inserted(target):
            shutil.copy(file, target)

        output = self.run_cli(["localsearch", "status", "--stat"])
        self.assertIn("nfo:Folder", output)
        self.assertIn("nfo:Document", output)
        self.assertIn("tracker:FileSystem", output)
        self.assertIn("tracker:Documents", output)

    def test_status_stat_term(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")

        # Copy a file and wait for it to be indexed, in order to ensure state
        file = datadir.joinpath("text/mango.txt")
        target = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file)))
        with self.await_document_inserted(target):
            shutil.copy(file, target)

        output = self.run_cli(["localsearch", "status", "--stat", "folder"])
        self.assertIn("nfo:Folder", output)
        self.assertNotIn("nfo:Document", output)

    def test_status_follow(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")
        output = ""
        with subprocess.Popen(
                ["localsearch", "status", "--follow"],
                bufsize=1,
                text=True,
                stdin=subprocess.DEVNULL,
                stdout=subprocess.PIPE,
                stderr=subprocess.DEVNULL) as proc:

            os.set_blocking(proc.stdout.fileno(), False)

            # Set up a temporary inhibition, this state should be reported in the output
            with subprocess.Popen(
                    ["localsearch", "inhibit", "cat"],
                    stdin=subprocess.PIPE,
                    stdout=subprocess.DEVNULL,
                    stderr=subprocess.DEVNULL) as inhibit_proc:

                n_attempts = 0
                while b'is paused' not in proc.stdout.buffer.peek():
                    time.sleep(1)
                    n_attempts += 1
                    assert n_attempts < 90

                # Close stdin to end the cat process
                inhibit_proc.stdin.close()

            # Copy a file and wait for it to be indexed
            file = datadir.joinpath("text/mango.txt")
            target = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file)))
            with self.await_document_inserted(target):
                shutil.copy(file, target)

            proc.terminate()
            output = proc.stdout.read()

        self.assertIn("is paused", output)
        self.assertIn("Idle", output)

    def test_status_watch(self):
        datadir = pathlib.Path(__file__).parent.joinpath("data/content")
        output = ""
        with subprocess.Popen(
                ["localsearch", "status", "--watch"],
                bufsize=1,
                text=True,
                stdin=subprocess.DEVNULL,
                stdout=subprocess.PIPE,
                stderr=subprocess.DEVNULL) as proc:

            # Copy a file and wait for it to be indexed
            file = datadir.joinpath("text/mango.txt")
            target = pathlib.Path(os.path.join(self.indexed_dir, os.path.basename(file)))
            with self.await_document_inserted(target):
                shutil.copy(file, target)

            time.sleep(2)

            proc.terminate()
            output = proc.stdout.read()

        # Check we've witnessed the file creation
        self.assertIn("mango.txt", output)
        self.assertIn("tracker:Documents", output)

    def test_status_wrongargs(self):
        err = None
        out = None
        try:
            # Pass unknown arg
            out = self.run_cli(["localsearch", "status", "--asdf"])
        except Exception as e:
            err = str(e)
        finally:
            self.assertIn("--asdf", err);
            self.assertIsNone(out)

if __name__ == "__main__":
    fixtures.tracker_test_main()
